package ca.jamdat.tetris;

import ca.jamdat.flight.*;
import ca.jamdat.fuser.*;
import ca.jamdat.flight.String;
import ca.jamdat.flight.Package;


public class BaseScene extends Scene {

public final static int arrowSelected = 0;
public final static int arrowPushed = 1;
public final static int arrowNormal = 2;
public final static int arrowDisabled = 3;

public final static int acceptSoftKey = 0;
public final static int declineSoftKey = 1;
public final static int softKeyCount = 2;

public final static int detached = Component.isDetached;
public final static int released = Selection.releasedState;
public final static int cancelled = Selection.canceledState	;

public final static int on = 0;
public final static int off = 1;

public final static int kCreated = 0;
public final static int kLoading = 1;
public final static int kLoaded = 2;
public final static int kWaitingOneOnTime = 3;
public final static int kEntranceAnimation = 4;
public final static int kDisplaying = 5;
public final static int kExitAnimation = 6;
public final static int kUnloading = 7;
public final static int kUnloaded = 8;

public  BaseScene (byte packageID) {
mState = (kCreated);
mTetrisState = GetApp().GetTetrisState();
mCommandUsedToInstantiate = ((((int)ConstantsEnglishPortrait.noneCommand == (int)ConstantsEnglishLandscape.noneCommand) ? Constants.noneCommand : ConstantsGetter.GetConstants().noneCommandFct()));
mPackageID = (packageID);



	SetExitSceneCommand((((int)ConstantsEnglishPortrait.noneCommand == (int)ConstantsEnglishLandscape.noneCommand) ? Constants.noneCommand : ConstantsGetter.GetConstants().noneCommandFct()));

	mSkipScene = false || (mPackageID == PackageID.emptyScene);

	SetRect(GetApp().GetRect());

	for(int i = acceptSoftKey;i < softKeyCount;i ++)
	{
		mpSoftKeySelection[i] = null;
		mpSoftKeyText[i] = null;
	}

}


public void destruct() {


}


public void SetCommandUsedToInstantiate(int command) {
 mCommandUsedToInstantiate = command; 
}


public int GetCommandUsedToInstantiate() {
 return  mCommandUsedToInstantiate; 
}


public boolean SupportsTransitionAnimation () {


	return  IsMenuScene() && !IsSplashMenuScenePackage();



}


public boolean MustDoTransitionAnimation () {


	return  SupportsTransitionAnimation() && !GetApp().IsHardwareOrientationChanging();



}


public boolean IsResultScene() {
 return  false; 
}


public boolean OnMsg (Component source, int msg, int intParam) {

	if(IsAppPausedWithFocus())
	{
		return  true;
	}

	
	if((msg == keyUpMsg || msg == keyDownOrRepeatMsg || msg == keyDownMsg) && mState != kDisplaying)
	{
		return  false;
	}

	if(msg == selectedMsg || msg == enabledMsg || msg == focusMsg || msg == pushedMsg || msg == attachMsg)
	{
		HandleSubtypeDisplay(source, msg, intParam);
	}


	 Vector2_short tempCoord2 = new Vector2_short(FlPenManager.Get().GetPenPosition(intParam));

 boolean retVal = false;


	
	 boolean isKeyUp = msg == keyUpMsg;

	if(isKeyUp || msg == keyDownMsg)
	{
		GetApp().OnKey((int)intParam, isKeyUp);
	}

	switch(msg)
	{
		case startTransitionMsg :	retVal = HandleStartTransitionMsg(source, msg, intParam);	break;

		case penDownMsg :
			{
				OnPenDown(tempCoord2);
				return  true;
			}
		case penUpMsg :
			{
				OnPenUp(tempCoord2);
				return  true;
			}
case attachMsg :				retVal = HandleAttachMsg(source, intParam);					break;
		case selectedMsg :			retVal = HandleSelectedMsg(source, intParam);				break;
		case pushedMsg :				retVal = HandlePushedMsg(source, intParam);					break;
		case enabledMsg :			retVal = HandleEnabledMsg(source, intParam);				break;
		case keyUpMsg :				retVal = OnKey((int)intParam, true);					break;
		case keyDownOrRepeatMsg :	retVal = OnKeyDownOrRepeat((int)intParam);			break;
		case keyDownMsg :			retVal = OnKey((int)intParam, false);					break;
		case focusMsg :				retVal = HandleFocusMsg(source, intParam);					break;
		case quitRequestMsg :		retVal = HandleQuitRequestMsg(source, intParam);			break;
		case quitTransitionMsg :	    retVal = HandleQuitTransitionMsg(source, intParam);			break;
		case slideMsg :				retVal = HandleSliderMsg(source, intParam);					break;
	}

	return  retVal;

}


public void OnTime (int totalTime, int deltaTime) {




	
	if(mpViewport == null && mpPackage != null && mpPackage.IsLoaded())
	{

		mState = kLoaded;
		GetResources();
		Init();

		if(IsSceneWithSoftKey())
		{
			InitSoftKeys();
		}
	}
	else if(mState == kLoaded)
	{
		mState = kWaitingOneOnTime;
	}
	else if(IsReadyToStartEntranceAnim())
	{
		mState = kEntranceAnimation;


		if(SupportsTransitionAnimation())
		{
			mpMenuWindowBorderTimerSequence.RegisterInGlobalTime();
		}

		StartEntranceAnimation();
		if(!MustDoTransitionAnimation())
		{
			StopEntranceAnimation();
		}
}

	if(mState == kEntranceAnimation && (IsEntranceAnimationOver()))
	{

		StopEntranceAnimation();
GiveFocusToScene();
		mState = kDisplaying;
		OnSceneReady();
	}

	if(mState == kExitAnimation && (IsExitAnimationOver()))
	{

		StopExitAnimation();
mState = kUnloading;
		ExitScene();
		return ;
	}


















}


public void OnNeedToReloadGame() {
 
}


public void Suspend () {

	Invalidate();


}


public void Resume () {

	Invalidate();

}


public void UpdateSoftKey (int softkey, String caption, int command) {

	 String tempString = new String();

	 boolean enabledState = (command == (((int)ConstantsEnglishPortrait.noneCommand == (int)ConstantsEnglishLandscape.noneCommand) ? Constants.noneCommand : ConstantsGetter.GetConstants().noneCommandFct()) || caption == null ? false : true);


	if(enabledState)
	{
		tempString.Assign(caption);
	}

	mpSoftKeyText[softkey].SetCaption(tempString);
	mpSoftKeySelection[softkey].SetEnabledState(enabledState);
	mpSoftKeySelection[softkey].SetCommand((byte)command);




}


public void EnableSoftKey (int softkey, boolean enableState) {

	if(enableState != mpSoftKeySelection[softkey].GetEnabledState())
	{
		mpSoftKeySelection[softkey].SetEnabledState(enableState);
		mpSoftKeySelection[softkey].SetVisible(enableState);




}

}


public boolean IsMenuScene () {

	
	return  true && mPackageID != PackageID.emptyScene;

}


public void OnPauseAppWithFocus () {





}


public void OnResumeAppWithFocus () {





}


public void OnIncomingCall() {
 
}


public byte GetPackageID() {
 return  mPackageID; 
}


public void SaveOnExit () {

	mSaveOnExit = true;

}


public boolean IsLoaded() {
 return  mState >= kLoaded && mState < kUnloading; 
}


public int GetExitSceneCommand() {
 return  mExitSceneTransitionCommand; 
}


public void SetExitSceneCommand(int cmd) {
 mExitSceneTransitionCommand = cmd; 
}


public void SetSkipOnBack(boolean skip) {
 mSkipScene = skip; 
}


public int GetReloadSceneCommand () {

	return  (((int)ConstantsEnglishPortrait.noneCommand == (int)ConstantsEnglishLandscape.noneCommand) ? Constants.noneCommand : ConstantsGetter.GetConstants().noneCommandFct());

}


public boolean HandleCommand (int command) {

	 boolean retval = true;







switch(command)
	{
		case (((int)ConstantsEnglishPortrait.chooseSelectedItemCommand == (int)ConstantsEnglishLandscape.chooseSelectedItemCommand) ? Constants.chooseSelectedItemCommand : -123) :
		{
			HandleCommand(GetFocusedSelectorCommand());
		}
		break;

		case (((int)ConstantsEnglishPortrait.artificialInterruptionCommand == (int)ConstantsEnglishLandscape.artificialInterruptionCommand) ? Constants.artificialInterruptionCommand : -123) :
		{

			GetApp().ArtificialInterruption();


}
		break;

		case (((int)ConstantsEnglishPortrait.singlePlayerModesAcceptCommand == (int)ConstantsEnglishLandscape.singlePlayerModesAcceptCommand) ? Constants.singlePlayerModesAcceptCommand : -123) :
		{
			HandleSingleGameModeSelectionCommand(true);
		}
		break;

		case Constants.marathonGameModeSelectedCommand :
		{
			 TetrisState tetrisState = GetApp().GetTetrisState();
			tetrisState.SetLocalGameMode(TetrisState.kModeMarathon);
			HandleSingleGameModeSelectionCommand(true);
		}
		break;

		case Constants.ultraGameModeSelectedCommand :
		{
			 TetrisState tetrisState = GetApp().GetTetrisState();
			tetrisState.SetLocalGameMode(TetrisState.kModeUltra);
			HandleSingleGameModeSelectionCommand(true);
		}	
		break;

		case Constants.fortyLinesGameModeSelectedCommand :
		{
			 TetrisState tetrisState = GetApp().GetTetrisState();
			tetrisState.SetLocalGameMode(TetrisState.kMode40Lines);
			HandleSingleGameModeSelectionCommand(true);
		}	
		break;
case (((int)ConstantsEnglishPortrait.singlePlayerModesDeclineCommand == (int)ConstantsEnglishLandscape.singlePlayerModesDeclineCommand) ? Constants.singlePlayerModesDeclineCommand : -123) :
		{
			HandleSingleGameModeSelectionCommand(false);
		}
		break;

		case (((int)ConstantsEnglishPortrait.selectGoToNextOptionCommand == (int)ConstantsEnglishLandscape.selectGoToNextOptionCommand) ? Constants.selectGoToNextOptionCommand : -123) :
		{
			 Selector selector = (Selector)mpComponentWithFocus;
			 Selector subSelector = (Selector)(selector.GetSelectionAt(selector.GetSingleSelection()).GetChild((((int)ConstantsEnglishPortrait.optionsSubSelectorChildIndex == (int)ConstantsEnglishLandscape.optionsSubSelectorChildIndex) ? Constants.optionsSubSelectorChildIndex : ConstantsGetter.GetConstants().optionsSubSelectorChildIndexFct())));

			subSelector.OnDefaultMsg(subSelector, Component.keyDownOrRepeatMsg, Key.RIGHT);
			subSelector.OnDefaultMsg(subSelector, Component.keyUpMsg, Key.RIGHT);
		}
		break;

        case (((int)ConstantsEnglishPortrait.exitGameCommand == (int)ConstantsEnglishLandscape.exitGameCommand) ? Constants.exitGameCommand : -123) :
        {

			FlPenManager.Get().Deactivate();
Application.Exit();
        }
        break;








case (((int)ConstantsEnglishPortrait.resetStatsCommand == (int)ConstantsEnglishLandscape.resetStatsCommand) ? Constants.resetStatsCommand : -123) :
			GetTetrisState().ResetStats();


case (((int)ConstantsEnglishPortrait.backFromOptionsMenuCommand == (int)ConstantsEnglishLandscape.backFromOptionsMenuCommand) ? Constants.backFromOptionsMenuCommand : -123) :
			


case (((int)ConstantsEnglishPortrait.goToSplashSceneCommand == (int)ConstantsEnglishLandscape.goToSplashSceneCommand) ? Constants.goToSplashSceneCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToMainMenuCommand == (int)ConstantsEnglishLandscape.goToMainMenuCommand) ? Constants.goToMainMenuCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToStatisticsMenuCommand == (int)ConstantsEnglishLandscape.goToStatisticsMenuCommand) ? Constants.goToStatisticsMenuCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToOptionsMenuCommand == (int)ConstantsEnglishLandscape.goToOptionsMenuCommand) ? Constants.goToOptionsMenuCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToOptionsMenuFromPauseCommand == (int)ConstantsEnglishLandscape.goToOptionsMenuFromPauseCommand) ? Constants.goToOptionsMenuFromPauseCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToRemapKeysSceneCommand == (int)ConstantsEnglishLandscape.goToRemapKeysSceneCommand) ? Constants.goToRemapKeysSceneCommand : -123) :

		case (((int)ConstantsEnglishPortrait.goToResetStatsPromptCommand == (int)ConstantsEnglishLandscape.goToResetStatsPromptCommand) ? Constants.goToResetStatsPromptCommand : -123) :
case (((int)ConstantsEnglishPortrait.goToGameSetupMenuSinglePlayerCommand == (int)ConstantsEnglishLandscape.goToGameSetupMenuSinglePlayerCommand) ? Constants.goToGameSetupMenuSinglePlayerCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToGameSetupMenuAgainstAICommand == (int)ConstantsEnglishLandscape.goToGameSetupMenuAgainstAICommand) ? Constants.goToGameSetupMenuAgainstAICommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToGameSetupMenuAgainstNetworkCommand == (int)ConstantsEnglishLandscape.goToGameSetupMenuAgainstNetworkCommand) ? Constants.goToGameSetupMenuAgainstNetworkCommand : -123) :
		case (((int)ConstantsEnglishPortrait.playAgainCommand == (int)ConstantsEnglishLandscape.playAgainCommand) ? Constants.playAgainCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToAboutMenuCommand == (int)ConstantsEnglishLandscape.goToAboutMenuCommand) ? Constants.goToAboutMenuCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToHelpMenuCommand == (int)ConstantsEnglishLandscape.goToHelpMenuCommand) ? Constants.goToHelpMenuCommand : -123) :

		case (((int)ConstantsEnglishPortrait.goToTutorialMenuCommand == (int)ConstantsEnglishLandscape.goToTutorialMenuCommand) ? Constants.goToTutorialMenuCommand : -123) :
		case (((int)ConstantsEnglishPortrait.startGameFromTutorialCommand == (int)ConstantsEnglishLandscape.startGameFromTutorialCommand) ? Constants.startGameFromTutorialCommand : -123) :
case (((int)ConstantsEnglishPortrait.selectDisplayOrientationCommand == (int)ConstantsEnglishLandscape.selectDisplayOrientationCommand) ? Constants.selectDisplayOrientationCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToDisplayOrientationMenuCommand == (int)ConstantsEnglishLandscape.goToDisplayOrientationMenuCommand) ? Constants.goToDisplayOrientationMenuCommand : -123) :
case (((int)ConstantsEnglishPortrait.goToPostScorePromptCommand == (int)ConstantsEnglishLandscape.goToPostScorePromptCommand) ? Constants.goToPostScorePromptCommand : -123) :
		case (((int)ConstantsEnglishPortrait.backCommand == (int)ConstantsEnglishLandscape.backCommand) ? Constants.backCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToOptionsMenuFromGameSetupCommand == (int)ConstantsEnglishLandscape.goToOptionsMenuFromGameSetupCommand) ? Constants.goToOptionsMenuFromGameSetupCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToGameSceneCommand == (int)ConstantsEnglishLandscape.goToGameSceneCommand) ? Constants.goToGameSceneCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToPauseSceneCommand == (int)ConstantsEnglishLandscape.goToPauseSceneCommand) ? Constants.goToPauseSceneCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToGameOverMenuCommand == (int)ConstantsEnglishLandscape.goToGameOverMenuCommand) ? Constants.goToGameOverMenuCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToReturnToMainPromptCommand == (int)ConstantsEnglishLandscape.goToReturnToMainPromptCommand) ? Constants.goToReturnToMainPromptCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToPlayAgainPromptCommand == (int)ConstantsEnglishLandscape.goToPlayAgainPromptCommand) ? Constants.goToPlayAgainPromptCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToExitPromptCommand == (int)ConstantsEnglishLandscape.goToExitPromptCommand) ? Constants.goToExitPromptCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToExitPromptFromPauseCommand == (int)ConstantsEnglishLandscape.goToExitPromptFromPauseCommand) ? Constants.goToExitPromptFromPauseCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToMoreGamesStaticCommand == (int)ConstantsEnglishLandscape.goToMoreGamesStaticCommand) ? Constants.goToMoreGamesStaticCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToMoreGamesPromptCommand == (int)ConstantsEnglishLandscape.goToMoreGamesPromptCommand) ? Constants.goToMoreGamesPromptCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToFreeStorageSpacePromptCommand == (int)ConstantsEnglishLandscape.goToFreeStorageSpacePromptCommand) ? Constants.goToFreeStorageSpacePromptCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToCorruptedSaveFilePromptCommand == (int)ConstantsEnglishLandscape.goToCorruptedSaveFilePromptCommand) ? Constants.goToCorruptedSaveFilePromptCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToGameSetupCommand == (int)ConstantsEnglishLandscape.goToGameSetupCommand) ? Constants.goToGameSetupCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToDelayLoadingScreenCommand == (int)ConstantsEnglishLandscape.goToDelayLoadingScreenCommand) ? Constants.goToDelayLoadingScreenCommand : -123) :
		case (((int)ConstantsEnglishPortrait.selectSinglePlayerMenuCommand == (int)ConstantsEnglishLandscape.selectSinglePlayerMenuCommand) ? Constants.selectSinglePlayerMenuCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToSingleGameSettingsCommand == (int)ConstantsEnglishLandscape.goToSingleGameSettingsCommand) ? Constants.goToSingleGameSettingsCommand : -123) :
		case (((int)ConstantsEnglishPortrait.startGameCommand == (int)ConstantsEnglishLandscape.startGameCommand) ? Constants.startGameCommand : -123) :
		case (((int)ConstantsEnglishPortrait.startGameAttractModeCommand == (int)ConstantsEnglishLandscape.startGameAttractModeCommand) ? Constants.startGameAttractModeCommand : -123) :
		case (((int)ConstantsEnglishPortrait.goToRemapDefaultsConfirmationPromptCommand == (int)ConstantsEnglishLandscape.goToRemapDefaultsConfirmationPromptCommand) ? Constants.goToRemapDefaultsConfirmationPromptCommand : -123) :











































{
			PrepareExitScene(command);
		}
		break;























default :
		{
			retval = false;
		}
		break;
	}
	return  retval;

}


public void OnReturnedFromLicenseActivation (int cmd) {





}


public void LoadResources () {



	mState = kLoading;

	mpPackage = RessourcesManager.Get().GetPackage(mPackageID);
	RessourcesManager.Get().AddPackageRef(mPackageID);






RegisterInGlobalTime();

}


public void GetResources () {

























if(mPackageID == PackageID.gameScene)
	{
		GetApp().GetPlayer().LoadGameSounds();
	}
mpPackage.SetNextEntryPointIndex(0);

	(mpViewport)= (mpViewport).Cast((mpPackage).GetEntryPoint(-1), null);



if(IsSceneWithSoftKey())
	{
		InitSoftKeysRessources();
	}







if(SupportsTransitionAnimation())
	{
		(mpMenuWindowAnimationTimeSystem)= (mpMenuWindowAnimationTimeSystem).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuWindowAnimTimeSystem_ID), null);
		(mpMenuWindowExitAnimationTimeSystem)= (mpMenuWindowExitAnimationTimeSystem).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuWindowExitAnimTimeSystem_ID), null);
		(mpMenuWindowBorderTimerSequence)= (mpMenuWindowBorderTimerSequence).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuWindowBorderTimerSequence_ID), null);

		
		if(mpMenuWindowExitAnimationTimeSystem.GetTotalTime() != 0)
		{
			mpMenuWindowExitAnimationTimeSystem.SetTotalTime(0);
			mpMenuWindowExitAnimationTimeSystem.OnTime(0, 0);
		}
	}
if(!IsSplashMenuScenePackage())
	{
		mpScramblingTextTimeSystem = new TimeSystem();
	}
mpViewport.SetViewport(this);

	GetApp().SetBackgroundVisible(true);
CompleteTransition();

}


public void UnloadResources () {




	GetApp().TakeFocus();
	if(GetPackageID() != PackageID.gameScene)
	{
		SetExitSceneCommand((((int)ConstantsEnglishPortrait.noneCommand == (int)ConstantsEnglishLandscape.noneCommand) ? Constants.noneCommand : ConstantsGetter.GetConstants().noneCommandFct()));
	}

	if(mpViewport != null && mpViewport.GetViewport() == this)
	{
		

		mpViewport.SetViewport(null);


		if(mState == kEntranceAnimation)
		{
			StopEntranceAnimation();
		}

		if(mState == kExitAnimation)
		{
			StopExitAnimation();
		}

		if(mpMenuWindowBorderTimerSequence != null)
		{
			mpMenuWindowBorderTimerSequence.UnRegisterInGlobalTime();
		}
}


	if(mpScramblingTextTimeSystem != null)
	{
		mpScramblingTextTimeSystem = null;
	}
mpViewport = null;

	mpComponentWithFocus = null;

	for(int i = acceptSoftKey;i < softKeyCount;i ++)
	{
		mpSoftKeySelection[i] = null;
		mpSoftKeyText[i] = null;
	}

	 RessourcesManager rm = RessourcesManager.Get();
	rm.RemovePackageRef(mPackageID);
	if(!rm.IsPackageReferenced(mPackageID))
	{
		rm.RemovePackageFromCache(mPackageID);
		rm.UnloadPackageDependencies(mPackageID);
	}

	mpPackage = null;

	UnRegisterInGlobalTime();

}


public void Init () {


	 TetrisApp tetrisApp = GetApp();


	if(mPackageID != PackageID.emptyScene)
{
		tetrisApp.SetLoadingSpriteVisible(false);
	}

	if(mPackageID == PackageID.aboutMenuScene)
	{
		{ 
			 Text aboutVersion = null;
			(aboutVersion)= (aboutVersion).Cast((mpPackage).GetEntryPoint(RSRC_aboutMenuScenePackage.aboutVersionText_ID), null);

			 String appVersion = new String(tetrisApp.GetJamdatBuildString());

			StringToolbox.AssignToUpper(appVersion);
aboutVersion.SetCaption(new String(appVersion));
			appVersion = null;
		}


		{ 
			 Viewport aboutScrollerViewport = null;
			(aboutScrollerViewport)= (aboutScrollerViewport).Cast((mpPackage).GetEntryPoint(RSRC_aboutMenuScenePackage.aboutScrollingViewport_ID), null);

			 short prevYBottom = 0;

			FlRect r = new FlRect();

			 Component c = null;
			
			 int aboutScrollerChildCount = aboutScrollerViewport.GetChildCount();

			for(int i = 0;i < aboutScrollerChildCount;i ++)
			{
				c = aboutScrollerViewport.GetChild(i);
				r.Assign(c.GetRect());

				r.MoveBy(0, (short)(prevYBottom - r.GetTop()));
				prevYBottom = r.GetBottom();
				c.SetRect(r);
			}
		}
}


	else if(mPackageID == PackageID.statisticsMenuScene)
	{
		
			(mUpArrowSelection)= (mUpArrowSelection).Cast((mpPackage).GetEntryPoint(RSRC_statisticsMenuScenePackage.statisticsMenuUpArrowSelection_ID), null);
			(mDownArrowSelection)= (mDownArrowSelection).Cast((mpPackage).GetEntryPoint(RSRC_statisticsMenuScenePackage.statisticsMenuDownArrowSelection_ID), null);
		for(int i = 0;i < TetrisState.kLocalPlayerStatsCount;i ++)
		{
			UpdateStatsItem(i);
		}

		 Selector statisticsMenuSelector = null;
		(statisticsMenuSelector)= (statisticsMenuSelector).Cast((mpPackage).GetEntryPoint(RSRC_statisticsMenuScenePackage.statisticsMenuSelector_ID), null);
		statisticsMenuSelector.SetSingleSelection(0);
	}
else if(mPackageID == PackageID.helpMenuScene)
	{	
		
			(mUpArrowSelection)= (mUpArrowSelection).Cast((mpPackage).GetEntryPoint(RSRC_helpMenuScenePackage.helpMenuUpArrowSelection_ID), null);
			(mDownArrowSelection)= (mDownArrowSelection).Cast((mpPackage).GetEntryPoint(RSRC_helpMenuScenePackage.helpMenuDownArrowSelection_ID), null);
		}
	else if(mPackageID == PackageID.marathonScoreMenuScene)
	{
		Scroller scrollerToReset = null;

		(scrollerToReset)= (scrollerToReset).Cast((mpPackage).GetEntryPoint(RSRC_marathonScoreMenuScenePackage.marathonScoreSceneScroller_ID), null);
		scrollerToReset.ResetScroller();
	}
	else if(mPackageID == PackageID.singlePlayerModesMenuScene)
	{
		GiveFocusToSelectedGameMode(mPackageID == PackageID.singlePlayerModesMenuScene);
	}

	else if(mPackageID == PackageID.tutorialMenuScene)
	{
		
			(mUpArrowSelection)= (mUpArrowSelection).Cast((mpPackage).GetEntryPoint(RSRC_tutorialMenuScenePackage.tutorialMenuUpArrowSelection_ID), null);
			(mDownArrowSelection)= (mDownArrowSelection).Cast((mpPackage).GetEntryPoint(RSRC_tutorialMenuScenePackage.tutorialMenuDownArrowSelection_ID), null);
		 TetrisState tetrisState = GetTetrisState();
		 int gameMode = tetrisState.GetGameMode();


		 Text tutorialContextText = null;
		 String tutorialContextString = null;
		(tutorialContextText)= (tutorialContextText).Cast((mpPackage).GetEntryPoint(RSRC_tutorialMenuScenePackage.tutorialMenuItemWrapped2Text_ID), null);
		int ressource;


		switch(gameMode)
		{
		case TetrisState.kModeMarathon :
			ressource = RSRC_tutorialMenuScenePackage.tutorialContentsMarathonString_ID;
			break;
		case TetrisState.kModeUltra :
			ressource = RSRC_tutorialMenuScenePackage.tutorialContentsUltraString_ID;
			break;
		case TetrisState.kMode40Lines :
			ressource = RSRC_tutorialMenuScenePackage.tutorialContents40LinesString_ID;
			break;




default :
			ressource = RSRC_tutorialMenuScenePackage.tutorialContentsMarathonString_ID;
			break;
		}

		(tutorialContextString)= (tutorialContextString).Cast((mpPackage).GetEntryPoint(ressource), null);

		
		tutorialContextText.SetCaption(new String(tutorialContextString));

		


















UpdateScene();
	}
else if(mPackageID == PackageID.displayOrientationMenuScene)
	{
		 Selector sel = null;
		(sel)= (sel).Cast((mpPackage).GetEntryPoint(RSRC_displayOrientationMenuScenePackage.displayOrientationSelector_ID), null);
		
		 ScreenManager sm = ScreenManager.Get();
		sel.SetSingleSelection(0);
		for(int i = ScreenManager.orientationOperationCount -1;i >= 0;i --)
		{
			 byte op = (byte)i;

			 byte resOri = sm.GetOrientationOperationResult(op);

			if(!sm.IsOrientationSettingSupported(resOri))
			{
				MenuToolbox.RemoveSelectionInVerticalSelector(sel, i);
			}
		}
	}
{
		tetrisApp.SetGameKeyAltMode(IsForcedNumlockNeeded());  
	}
if(SupportsTransitionAnimation())
	{
		if(IsSceneWithCursor())
		{
			InitCursorShow();
		}

		mpMenuWindowAnimationTimeSystem.SetTotalTime(0);
		mpMenuWindowAnimationTimeSystem.OnTime(0, 0);
	}
if(IsSceneWithScramblingAnimation() && MustDoTransitionAnimation())
	{
		 Array_int scrambleMenuTimeSystemArray = null;
		 Package p = RessourcesManager.Get().GetPackage(PackageID.common);
		(scrambleMenuTimeSystemArray)= (scrambleMenuTimeSystemArray).Cast((p).GetEntryPoint(RSRC_commonPackage.scramblingTextMenuTimePeriod_ID), null);

		 int scramblingPeriodIndex = mPackageID - PackageID.gameScene;

		 Font scramblingFont = GetApp().GetScramblingFont();

		 Text textToScramble = null;

		if(scramblingPeriodIndex > 0)
		{
			mScramblingTextDuration = scrambleMenuTimeSystemArray.GetAt(scramblingPeriodIndex);
			 int selectionCount = ((Selector)mpComponentWithFocus).GetNumSelections();

			 int selectionIndex = 0;

			 Selection currentSelection = null;
			 int itemSubtype = -1;


			for(int i = 0;i < selectionCount;i ++)
			{

				currentSelection = ((Selection)((Selector)mpComponentWithFocus).GetSelectionAt(i));
				textToScramble = (Text)currentSelection.GetChild(0);

				if((textToScramble instanceof Text))
				{
					itemSubtype = currentSelection.GetSubtype();

					mpScramblingTextTimeSystem.Register(new ScramblingText(textToScramble, scramblingFont, mScramblingTextDuration));
					if(currentSelection.GetChildCount() > 1 &&
						itemSubtype != (((int)ConstantsEnglishPortrait.scrollerInSelectorSubtype == (int)ConstantsEnglishLandscape.scrollerInSelectorSubtype) ? Constants.scrollerInSelectorSubtype : ConstantsGetter.GetConstants().scrollerInSelectorSubtypeFct()) &&
						itemSubtype != (((int)ConstantsEnglishPortrait.selectorInSelectorSubtype == (int)ConstantsEnglishLandscape.selectorInSelectorSubtype) ? Constants.selectorInSelectorSubtype : ConstantsGetter.GetConstants().selectorInSelectorSubtypeFct()) &&
						itemSubtype != (((int)ConstantsEnglishPortrait.textfieldItemSelectionSubtype == (int)ConstantsEnglishLandscape.textfieldItemSelectionSubtype) ? Constants.textfieldItemSelectionSubtype : ConstantsGetter.GetConstants().textfieldItemSelectionSubtypeFct()) &&
						itemSubtype != (((int)ConstantsEnglishPortrait.textfieldItemSubSelectionSubtype == (int)ConstantsEnglishLandscape.textfieldItemSubSelectionSubtype) ? Constants.textfieldItemSubSelectionSubtype : ConstantsGetter.GetConstants().textfieldItemSubSelectionSubtypeFct()) &&
						itemSubtype != (((int)ConstantsEnglishPortrait.menuItemSelectionWithSubSelectorAndCursorSubtype == (int)ConstantsEnglishLandscape.menuItemSelectionWithSubSelectorAndCursorSubtype) ? Constants.menuItemSelectionWithSubSelectorAndCursorSubtype : ConstantsGetter.GetConstants().menuItemSelectionWithSubSelectorAndCursorSubtypeFct()) &&
						(currentSelection.GetChild(1) instanceof Selector))
					{
						selectionIndex = ((Selector)(currentSelection.GetChild(1))).GetSingleSelection();
						if(selectionIndex >= 0)
						{
							textToScramble = (Text)((Selection)((Selector)(currentSelection.GetChild(1))).GetSelectionAt(selectionIndex)).GetChild(0);
							mpScramblingTextTimeSystem.Register(new ScramblingText(textToScramble, scramblingFont, mScramblingTextDuration));
						}
					}
				}
			}
		}
		else
		{
			mScramblingTextDuration = (((int)ConstantsEnglishPortrait.animScramblingTextTitlePeriod == (int)ConstantsEnglishLandscape.animScramblingTextTitlePeriod) ? Constants.animScramblingTextTitlePeriod : ConstantsGetter.GetConstants().animScramblingTextTitlePeriodFct());
		}

		(textToScramble)= (textToScramble).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuTitleText_ID), null);
		mpScramblingTextTimeSystem.Register(new ScramblingText(textToScramble, scramblingFont, (((int)ConstantsEnglishPortrait.animScramblingTextTitlePeriod == (int)ConstantsEnglishLandscape.animScramblingTextTitlePeriod) ? Constants.animScramblingTextTitlePeriod : ConstantsGetter.GetConstants().animScramblingTextTitlePeriodFct())));


	}

}


public void UpdateStatsItem (int indexOfItem) {

	 Text statsText = null;
	(statsText)= (statsText).Cast((mpPackage).GetEntryPoint(RSRC_statisticsMenuScenePackage.statisticsMenuSingleItem3Text_ID + indexOfItem), null);

	 int currentStats = GetTetrisState().GetStatistic(indexOfItem);

	String usedCaption = null;

	if(indexOfItem == TetrisState.kStat40LinesTime)
	{
		 String formatedTime = new String(StringToolbox.FormatTime(currentStats * 1000));

		usedCaption = new String(formatedTime);
	}
	else
	{
		usedCaption = new String(currentStats);
	}
	statsText.SetCaption(usedCaption);

}


public void InitSoftKeys () {

	







if(mPackageID == PackageID.gameSetupMenuScene && !GetApp().IsInGameLoadingFlow())
	{
		 String okSoftKeyStr = null;
		(okSoftKeyStr)= (okSoftKeyStr).Cast((mpPackage).GetEntryPoint(RSRC_gameSetupMenuScenePackage.okSoftKeyString_ID), null);

		
		UpdateSoftKey(acceptSoftKey, okSoftKeyStr, (((int)ConstantsEnglishPortrait.goToOptionsMenuFromGameSetupCommand == (int)ConstantsEnglishLandscape.goToOptionsMenuFromGameSetupCommand) ? Constants.goToOptionsMenuFromGameSetupCommand : ConstantsGetter.GetConstants().goToOptionsMenuFromGameSetupCommandFct()));
	}


}


public void InitSoftKeysRessources () {

	(mpComponentWithFocus)= (mpComponentWithFocus).Cast((mpPackage).GetEntryPoint(-1), null);

	
	for(int i = acceptSoftKey;i < softKeyCount;i ++)
	{
		(mpSoftKeySelection[i])= (mpSoftKeySelection[i]).Cast((mpPackage).GetEntryPoint(-1), null);




		mpSoftKeyText[i] = (Text)mpSoftKeySelection[i].GetChild((((int)ConstantsEnglishPortrait.softkeySelectionsTextChildIndex == (int)ConstantsEnglishLandscape.softkeySelectionsTextChildIndex) ? Constants.softkeySelectionsTextChildIndex : ConstantsGetter.GetConstants().softkeySelectionsTextChildIndexFct()));
	}













{
		 short sceneWidth = (short)(((int)ConstantsEnglishPortrait.screenWidth == (int)ConstantsEnglishLandscape.screenWidth) ? Constants.screenWidth : ConstantsGetter.GetConstants().screenWidthFct());

		 short sceneHeight = (short)(((int)ConstantsEnglishPortrait.screenHeight == (int)ConstantsEnglishLandscape.screenHeight) ? Constants.screenHeight : ConstantsGetter.GetConstants().screenHeightFct());

		 short softKeyHeight = mpSoftKeySelection[0].GetRectHeight();

		 short bottomSoftKeyTop = (short)(sceneHeight - softKeyHeight);

		 short softKeyTextLeft = (short)(((int)ConstantsEnglishPortrait.rotatedSoftKeyTextPixelXOffset == (int)ConstantsEnglishLandscape.rotatedSoftKeyTextPixelXOffset) ? Constants.rotatedSoftKeyTextPixelXOffset : ConstantsGetter.GetConstants().rotatedSoftKeyTextPixelXOffsetFct());

		 short softKeyTextWidth = (short)(sceneWidth - softKeyTextLeft * 2);

		 short softKeyTextYOffset = (short)(((int)ConstantsEnglishPortrait.rotatedSoftKeyTextPixelYOffset == (int)ConstantsEnglishLandscape.rotatedSoftKeyTextPixelYOffset) ? Constants.rotatedSoftKeyTextPixelYOffset : ConstantsGetter.GetConstants().rotatedSoftKeyTextPixelYOffsetFct());


		
		

		 int left = 0;

		 int right = 1;

		byte[] softKeyTextAlign = new byte[softKeyCount];
		boolean[] softKeyAtTop = new boolean[softKeyCount];















softKeyTextAlign[left] = Alignment.LEFT;
		softKeyAtTop[left] = false;
		softKeyTextAlign[right] = Alignment.RIGHT;
		softKeyAtTop[right] = false;







































for(int i = 0;i < (softKeyCount);i ++)
		{
			 boolean atTop = softKeyAtTop[i];

			 byte align = softKeyTextAlign[i];


			 short textYOffset = softKeyTextYOffset;

			 short rect_left = 0;
  short rect_top = bottomSoftKeyTop;
  short rect_width = sceneWidth;
  short rect_height = softKeyHeight;


			if(atTop)
			{
				rect_top = 0;
				textYOffset = (short)-softKeyTextYOffset;
			}

			
			mpSoftKeySelection[i].SetRect(rect_left, rect_top, rect_width, rect_height);

			
			rect_left = softKeyTextLeft;
			rect_width = softKeyTextWidth;
			rect_top = textYOffset;
			
			mpSoftKeyText[i].SetRect(rect_left, rect_top, rect_width, rect_height);
			mpSoftKeyText[i].SetAlignment(align);



}
	}

	 boolean showSoftKeys = true;


	
	

TetrisApp.Get().SetSoftKeySpriteVisible(showSoftKeys);
	mpSoftKeySelection[acceptSoftKey].SetVisible(showSoftKeys);
	mpSoftKeySelection[declineSoftKey].SetVisible(showSoftKeys);


}


public void GiveFocusToScene () {

	
	if(mpComponentWithFocus != null)
	{
		mpComponentWithFocus.TakeFocus();
	}

}


public void GiveFocusToSelectedGameMode (boolean localGame) {




	 Selector gameModeSelector = (Selector)mpComponentWithFocus;
	 int selectionIndex = -1;


	if(localGame)
	{


		 int gameMode = GetTetrisState().GetLocalGameMode();

		selectionIndex = gameMode;







}













	gameModeSelector.SetSingleSelection(selectionIndex);

}


public void PrepareExitScene (int command) {



	
	SetExitSceneCommand(command);
	GetApp().TakeFocus();



if(command == (((int)ConstantsEnglishPortrait.selectDisplayOrientationCommand == (int)ConstantsEnglishLandscape.selectDisplayOrientationCommand) ? Constants.selectDisplayOrientationCommand : ConstantsGetter.GetConstants().selectDisplayOrientationCommandFct()) && mPackageID != PackageID.emptyScene)
	{
		 ScreenManager sm = ScreenManager.Get();
		 int toggleOptionIndex = ((Selector)mpComponentWithFocus).GetSingleSelection();

		 byte op = (byte)toggleOptionIndex;

		 byte currentVal = sm.GetOrientationSetting();

		 byte newVal = sm.GetOrientationOperationResult(op);

		if(currentVal != newVal)
		{
			sm.SetOrientationSetting(newVal);
		}
	}
if(mState < kExitAnimation) 
	{

		if(mState == kEntranceAnimation)
		{
			StopEntranceAnimation();
		}
if(mState >= kLoaded) 
		{
			mState = kExitAnimation;

			if(IsMenuScene())
			{
				StartExitAnimation();
			}

if(mPackageID == PackageID.gameScene)
			{

				UnloadResources();
				mState = kUnloaded;
				GetApp().TakeFocus();
				ExitScene();
			}
		}
		else  
		{
			
			
			if(mState == kLoading)
			{
				UnloadResources();
			}
			

			CompleteTransition();
			mState = kUnloaded;
			GetApp().TakeFocus();
			ExitScene();
			return ;
		}
	}
	else if(mState > kExitAnimation)
	{
		ExitScene();
	}

}


public void ExitScene () {



	 TetrisApp tetrisApp = GetApp();

	if(mSaveOnExit)
	{
		tetrisApp.SetMustSave();
		mSaveOnExit = false;
	}

	
	tetrisApp.HandleSceneTransitionCommand(GetExitSceneCommand());

}


public void UpdateScene () {

	 Selector selector = ((Selector)mpComponentWithFocus);
	if(selector.GetSingleSelection() != -1)
	{
		 Scroller scroller = (Scroller)(selector.GetSelectionAt(selector.GetSingleSelection()).GetChild((((int)ConstantsEnglishPortrait.helpScrollerChildIndex == (int)ConstantsEnglishLandscape.helpScrollerChildIndex) ? Constants.helpScrollerChildIndex : ConstantsGetter.GetConstants().helpScrollerChildIndexFct())));

		scroller.ResetScroller();
	}

}


public boolean OnKey (int key, boolean up) {

	 boolean retval = true;


	if(up)
	{
	












}

	switch(key)
	{
		

		case Key.CLEAR :
		{
			if(mpSoftKeySelection[declineSoftKey] != null && mpSoftKeySelection[declineSoftKey].GetEnabledState())
			{
				mpSoftKeySelection[declineSoftKey].SetPushedState(!up);
			}
		}
		break;

		
		case Key.SOFT_SELECT :	
		case Key.FIRE :
		{
			if(mpSoftKeySelection[acceptSoftKey] != null && mpSoftKeySelection[acceptSoftKey].GetEnabledState() && IsKeyMappedToAcceptSoftkey(key))
			{
				mpSoftKeySelection[acceptSoftKey].SetPushedState(!up);
			}
		}
		break;

		default :
		{
			retval = false;
		}
		break;
	}
	return  retval;

}


public boolean OnKeyDownOrRepeat (int key) {


	return  false;

}


public int GetFocusedSelectorCommand () {



	 Selector selectorWithFocus = ((Selector)mpComponentWithFocus);
	 int command = selectorWithFocus.GetSelectionAt(selectorWithFocus.GetSingleSelection()).GetCommand();

	return  command;

}


public void SaveSelectedGameModeToState () {

	

	 TetrisState tetrisState = GetTetrisState();


int gameMode;


	switch(GetFocusedSelectorCommand())
	{
		case (((int)ConstantsEnglishPortrait.marathonGameModeSelectedCommand == (int)ConstantsEnglishLandscape.marathonGameModeSelectedCommand) ? Constants.marathonGameModeSelectedCommand : -123) :
			gameMode = TetrisState.kModeMarathon;
			break;
		case (((int)ConstantsEnglishPortrait.ultraGameModeSelectedCommand == (int)ConstantsEnglishLandscape.ultraGameModeSelectedCommand) ? Constants.ultraGameModeSelectedCommand : -123) :
			gameMode = TetrisState.kModeUltra;
			break;
		case (((int)ConstantsEnglishPortrait.fortyLinesGameModeSelectedCommand == (int)ConstantsEnglishLandscape.fortyLinesGameModeSelectedCommand) ? Constants.fortyLinesGameModeSelectedCommand : -123) :
			gameMode = TetrisState.kMode40Lines;
			break;









default :

			gameMode = TetrisState.kModeCount;
	}








{
		tetrisState.SetLocalGameMode(gameMode);
	}

}


public void HandleSingleGameModeSelectionCommand (boolean isAcceptSoftKey) {




if(isAcceptSoftKey)
	{
		





{
			SaveSelectedGameModeToState(); 
			HandleCommand((((int)ConstantsEnglishPortrait.goToGameSetupCommand == (int)ConstantsEnglishLandscape.goToGameSetupCommand) ? Constants.goToGameSetupCommand : ConstantsGetter.GetConstants().goToGameSetupCommandFct()));
		}
	}
	else
	{
		HandleCommand((((int)ConstantsEnglishPortrait.backCommand == (int)ConstantsEnglishLandscape.backCommand) ? Constants.backCommand : ConstantsGetter.GetConstants().backCommandFct()));
	}

}


public boolean HandleStartTransitionMsg (Component source, int msg, int intParam) {

	if(intParam == (((int)ConstantsEnglishPortrait.automaticDismissCommand == (int)ConstantsEnglishLandscape.automaticDismissCommand) ? Constants.automaticDismissCommand : ConstantsGetter.GetConstants().automaticDismissCommandFct()) || mPackageID < PackageID.invalid)
	{
		
		CompleteTransition();

		mState = kUnloaded;
		GetApp().TakeFocus();

		if(mPackageID < PackageID.invalid)
		{
			mPackageID = (byte)(-mPackageID);
		}
		return  true;
	}







if(mSkipScene && intParam == 0)
	{
		DetachAndSkip(source, msg, intParam);
	}
	else
	{
		GetApp().SetLoadingSpriteVisible(true);
		LoadResources();
	}
	return  true;

}


public boolean HandleAttachMsg (Component source, int intParam) {

	if(detached == intParam)
	{
		
		if(source == this && mState >= kLoading && mState < kUnloaded)
		{
			UnloadResources();
			mState = kUnloaded;
        }
		return  true;
	}
	return  false;

}


public boolean HandleSelectedMsg (Component source, int intParam) {



	return  false;

}


public boolean HandlePushedMsg (Component source, int intParam) {

	if(intParam == released || intParam == cancelled)
	{	
		 int cmd = ((Selection)source).GetCommand();


	






{
			HandleCommand(cmd);
			return  true;
		}
	}
	return  false;

}


public boolean HandleEnabledMsg (Component source, int intParam) {



	return  false;

}


public boolean HandleFocusMsg (Component source, int intParam) {



	return  false;

}


public boolean HandleQuitRequestMsg (Component source, int intParam) {




	
	
	Application.Exit();

	
	return  false;

}


public boolean HandleQuitTransitionMsg (Component source, int intParam) {




	if(mState >= kLoading && mState < kUnloaded)
	{
		UnloadResources();
		mState = kUnloaded;
	}

	
	return  false;

}


public boolean HandleSubtypeDisplay (Component source, int msg, int intParam) {

	 boolean retval = false;


	if ((source instanceof Selector))
	{
		 Selector selector = (Selector)source;
		 int subtype = selector.GetSubtype();


		if(subtype == (((int)ConstantsEnglishPortrait.fowardFocusSelectorSubtype == (int)ConstantsEnglishLandscape.fowardFocusSelectorSubtype) ? Constants.fowardFocusSelectorSubtype : ConstantsGetter.GetConstants().fowardFocusSelectorSubtypeFct()) && msg != attachMsg && intParam == 1)
		{
			selector.SetSingleSelection(0);
		}
	}
	else if ((source instanceof Selection))
	{
		 Selection selection = (Selection)source;
		 boolean isSelected = selection.GetSelectedState();


		 boolean isEnabled = selection.GetEnabledState();

		 boolean isPushed = selection.GetPushedState();

		retval = true;

         int subtype = selection.GetSubtype();

		switch(subtype)
		{
			case (((int)ConstantsEnglishPortrait.menuItemWithCursorSelectionSubtype == (int)ConstantsEnglishLandscape.menuItemWithCursorSelectionSubtype) ? Constants.menuItemWithCursorSelectionSubtype : -123) :
            case (((int)ConstantsEnglishPortrait.menuSimpleItemSelectionSubtype == (int)ConstantsEnglishLandscape.menuSimpleItemSelectionSubtype) ? Constants.menuSimpleItemSelectionSubtype : -123) :
			case (((int)ConstantsEnglishPortrait.menuSimpleItemSelectionSubtypeEnabledWhenHasCommand == (int)ConstantsEnglishLandscape.menuSimpleItemSelectionSubtypeEnabledWhenHasCommand) ? Constants.menuSimpleItemSelectionSubtypeEnabledWhenHasCommand : -123) :
			case (((int)ConstantsEnglishPortrait.menuSpriteItemSelectionSubtype == (int)ConstantsEnglishLandscape.menuSpriteItemSelectionSubtype) ? Constants.menuSpriteItemSelectionSubtype : -123) :
			case (((int)ConstantsEnglishPortrait.selectorInSelectorSubtype == (int)ConstantsEnglishLandscape.selectorInSelectorSubtype) ? Constants.selectorInSelectorSubtype : -123) :
			{
				OnMenuItemSelectionSubtypeUIEvent(selection, isEnabled, isSelected, subtype);
            }
			break;






case (((int)ConstantsEnglishPortrait.scrollerInSelectorSubtype == (int)ConstantsEnglishLandscape.scrollerInSelectorSubtype) ? Constants.scrollerInSelectorSubtype : -123) :
			{
				
				UpdateScene();
			}
		    break;

            case (((int)ConstantsEnglishPortrait.menuSelectorArrowSubtype == (int)ConstantsEnglishLandscape.menuSelectorArrowSubtype) ? Constants.menuSelectorArrowSubtype : -123) :
			{
				OnArrowPushedSubtype(selection, isSelected, isEnabled, isPushed);
			}
			break;

			
            case (((int)ConstantsEnglishPortrait.menuScrollerArrowSubtype == (int)ConstantsEnglishLandscape.menuScrollerArrowSubtype) ? Constants.menuScrollerArrowSubtype : -123) :
			{
				OnArrowPushedSubtype(selection, false, isEnabled, isPushed);
			}
			break;

			case (((int)ConstantsEnglishPortrait.menuItemSelectionWithSubSelectorAndCursorSubtype == (int)ConstantsEnglishLandscape.menuItemSelectionWithSubSelectorAndCursorSubtype) ? Constants.menuItemSelectionWithSubSelectorAndCursorSubtype : -123) :
            case (((int)ConstantsEnglishPortrait.menuItemSelectionWithSubSelectorSubtype == (int)ConstantsEnglishLandscape.menuItemSelectionWithSubSelectorSubtype) ? Constants.menuItemSelectionWithSubSelectorSubtype : -123) :
			{
                OnMenuItemSelectionWithSubSelectorSubtypeUIEvent(selection, isEnabled, isSelected);
			}
            break;

			default :
			{
				retval = false;
			}
			break;
 		}
	}

	return  retval;

}


public void OnMenuItemSelectionSubtypeUIEvent (Selection selection, boolean enable, boolean selected, int subtype) {


























{
		if(subtype != (((int)ConstantsEnglishPortrait.menuSpriteItemSelectionSubtype == (int)ConstantsEnglishLandscape.menuSpriteItemSelectionSubtype) ? Constants.menuSpriteItemSelectionSubtype : ConstantsGetter.GetConstants().menuSpriteItemSelectionSubtypeFct()))
		{
			selection.SetVisible(enable);
		}
	}

    if(enable)
    {
		
		if (subtype == (((int)ConstantsEnglishPortrait.menuSimpleItemSelectionSubtype == (int)ConstantsEnglishLandscape.menuSimpleItemSelectionSubtype) ? Constants.menuSimpleItemSelectionSubtype : ConstantsGetter.GetConstants().menuSimpleItemSelectionSubtypeFct()) ||
			subtype == (((int)ConstantsEnglishPortrait.menuSimpleItemSelectionSubtypeEnabledWhenHasCommand == (int)ConstantsEnglishLandscape.menuSimpleItemSelectionSubtypeEnabledWhenHasCommand) ? Constants.menuSimpleItemSelectionSubtypeEnabledWhenHasCommand : ConstantsGetter.GetConstants().menuSimpleItemSelectionSubtypeEnabledWhenHasCommandFct()) ||
			subtype == (((int)ConstantsEnglishPortrait.submenuItemOfSelectionWithSubSelectorSubtype == (int)ConstantsEnglishLandscape.submenuItemOfSelectionWithSubSelectorSubtype) ? Constants.submenuItemOfSelectionWithSubSelectorSubtype : ConstantsGetter.GetConstants().submenuItemOfSelectionWithSubSelectorSubtypeFct()) ||
			subtype == (((int)ConstantsEnglishPortrait.menuItemSelectionWithSubSelectorAndCursorSubtype == (int)ConstantsEnglishLandscape.menuItemSelectionWithSubSelectorAndCursorSubtype) ? Constants.menuItemSelectionWithSubSelectorAndCursorSubtype : ConstantsGetter.GetConstants().menuItemSelectionWithSubSelectorAndCursorSubtypeFct()) ||
			subtype == (((int)ConstantsEnglishPortrait.menuItemSelectionWithSubSelectorSubtype == (int)ConstantsEnglishLandscape.menuItemSelectionWithSubSelectorSubtype) ? Constants.menuItemSelectionWithSubSelectorSubtype : ConstantsGetter.GetConstants().menuItemSelectionWithSubSelectorSubtypeFct()) ||
			subtype == (((int)ConstantsEnglishPortrait.menuItemWithCursorSelectionSubtype == (int)ConstantsEnglishLandscape.menuItemWithCursorSelectionSubtype) ? Constants.menuItemWithCursorSelectionSubtype : ConstantsGetter.GetConstants().menuItemWithCursorSelectionSubtypeFct()) ||
			subtype == (((int)ConstantsEnglishPortrait.textfieldItemSelectionSubtype == (int)ConstantsEnglishLandscape.textfieldItemSelectionSubtype) ? Constants.textfieldItemSelectionSubtype : ConstantsGetter.GetConstants().textfieldItemSelectionSubtypeFct()))
		{
			ChangeFont((Text)selection.GetChild((((int)ConstantsEnglishPortrait.menuItemTextChildIndex == (int)ConstantsEnglishLandscape.menuItemTextChildIndex) ? Constants.menuItemTextChildIndex : ConstantsGetter.GetConstants().menuItemTextChildIndexFct())), enable, selected, subtype == (((int)ConstantsEnglishPortrait.submenuItemOfSelectionWithSubSelectorSubtype == (int)ConstantsEnglishLandscape.submenuItemOfSelectionWithSubSelectorSubtype) ? Constants.submenuItemOfSelectionWithSubSelectorSubtype : ConstantsGetter.GetConstants().submenuItemOfSelectionWithSubSelectorSubtypeFct()));
		}

		
		
        if (selected && IsSceneWithCursor() && (subtype == (((int)ConstantsEnglishPortrait.menuItemWithCursorSelectionSubtype == (int)ConstantsEnglishLandscape.menuItemWithCursorSelectionSubtype) ? Constants.menuItemWithCursorSelectionSubtype : ConstantsGetter.GetConstants().menuItemWithCursorSelectionSubtypeFct()) ||
			subtype == (((int)ConstantsEnglishPortrait.menuItemSelectionWithSubSelectorAndCursorSubtype == (int)ConstantsEnglishLandscape.menuItemSelectionWithSubSelectorAndCursorSubtype) ? Constants.menuItemSelectionWithSubSelectorAndCursorSubtype : ConstantsGetter.GetConstants().menuItemSelectionWithSubSelectorAndCursorSubtypeFct())))
        {
			UpdateCursorPosition(selection);
        }

		if(subtype == (((int)ConstantsEnglishPortrait.selectorInSelectorSubtype == (int)ConstantsEnglishLandscape.selectorInSelectorSubtype) ? Constants.selectorInSelectorSubtype : ConstantsGetter.GetConstants().selectorInSelectorSubtypeFct()) && selected)
		{
			IndexedSprite cursor = null;


			(cursor)= (cursor).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuCursorIndexedSprite_ID), null);

			cursor.SetVisible(false);
		}
    }

}


public void UpdateCursorPosition (Selection selection) {

	IndexedSprite cursor = null;

	 Viewport pMenuViewport = null;

	(cursor)= (cursor).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuCursorIndexedSprite_ID), null);
	(pMenuViewport)= (pMenuViewport).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuScrollingViewport_ID), null);

	
	 Text pMenuText = null;
	pMenuText = (Text)selection.GetChild((((int)ConstantsEnglishPortrait.menuItemTextChildIndex == (int)ConstantsEnglishLandscape.menuItemTextChildIndex) ? Constants.menuItemTextChildIndex : ConstantsGetter.GetConstants().menuItemTextChildIndexFct()));

	 short xCenter = (short)(pMenuText.GetAbsoluteLeft() + pMenuText.GetRectWidth() / 2);

	 short textWidth = (short)(pMenuText.GetFont().GetLineWidth(pMenuText.GetCaption()));


	 short xAlign = (short)(xCenter - textWidth / 2- (((int)ConstantsEnglishPortrait.cursorOffsetX == (int)ConstantsEnglishLandscape.cursorOffsetX) ? Constants.cursorOffsetX : ConstantsGetter.GetConstants().cursorOffsetXFct()));


	 short yAlign = (short)(selection.GetTopLeft().GetY() + (((int)ConstantsEnglishPortrait.cursorOffsetY == (int)ConstantsEnglishLandscape.cursorOffsetY) ? Constants.cursorOffsetY : ConstantsGetter.GetConstants().cursorOffsetYFct()) - pMenuViewport.GetOffsetY());


	cursor.SetTopLeft(xAlign, yAlign);

}


public void OnMenuItemSelectionWithSubSelectorSubtypeUIEvent (Selection selection, boolean isEnabled, boolean isSelected) {

	 Selector subSelector = (Selector)selection.GetChild((((int)ConstantsEnglishPortrait.optionsSubSelectorChildIndex == (int)ConstantsEnglishLandscape.optionsSubSelectorChildIndex) ? Constants.optionsSubSelectorChildIndex : ConstantsGetter.GetConstants().optionsSubSelectorChildIndexFct()));
	 Viewport subSelectorViewport = (Viewport)subSelector.GetChild((((int)ConstantsEnglishPortrait.optionsSubSelectorViewportChildIndex == (int)ConstantsEnglishLandscape.optionsSubSelectorViewportChildIndex) ? Constants.optionsSubSelectorViewportChildIndex : ConstantsGetter.GetConstants().optionsSubSelectorViewportChildIndexFct()));

	 int subtype = selection.GetSubtype();

	OnMenuItemSelectionSubtypeUIEvent(selection, isEnabled, isSelected, subtype);

	for(int i = 0;i < subSelectorViewport.GetChildCount();i ++)
	{
		 Selection subSelection = (Selection)subSelectorViewport.GetChild(i);
		subtype = subSelection.GetSubtype();
		OnMenuItemSelectionSubtypeUIEvent(subSelection, isEnabled, isSelected, subtype);
	}

}


public void OnArrowPushedSubtype (Selection selection, boolean selected, boolean enabled, boolean pushed) {

	 IndexedSprite arrow = (IndexedSprite)selection.GetChild((((int)ConstantsEnglishPortrait.menuArrowChildIndex == (int)ConstantsEnglishLandscape.menuArrowChildIndex) ? Constants.menuArrowChildIndex : ConstantsGetter.GetConstants().menuArrowChildIndexFct()));

	if(enabled)
	{
		if(selected)
		{
			arrow.SetCurrentFrame(pushed ? arrowPushed : arrowSelected);
		}
		else
		{
			arrow.SetCurrentFrame(pushed ? arrowPushed : arrowNormal);
		}
	}
	else
	{
		arrow.SetCurrentFrame(arrowDisabled);
	}

}


public void ChangeFont (Text currentText, boolean enable, boolean selected, boolean isSubSelection) {





	 Package fontPackage = RessourcesManager.Get().GetPackage(PackageID.font);
	 Font currentFont = null;







{
		if(selected)
		{
			(currentFont)= (currentFont).Cast((fontPackage).GetEntryPoint(RSRC_fontPackage.menuItemSelectedFont_ID), null);
		}
		else
		{
			if(isSubSelection)
			{
				(currentFont)= (currentFont).Cast((fontPackage).GetEntryPoint(RSRC_fontPackage.menuSubSelectionFont_ID), null);
			}
			else
			{
				(currentFont)= (currentFont).Cast((fontPackage).GetEntryPoint(RSRC_fontPackage.menuItemNormalFont_ID), null);
			}
			
		}		
	}


	currentText.SetFont(currentFont);

}


public boolean HandleSliderMsg(Component a13, int a12) {
 return  false; 
}


public void OnMenuItemSelectionWithSliderSubtypeUIEvent(Selection a16, boolean a15, boolean a14) {
 
}


public void UpdateSlider(Slider a17) {
 
}


public boolean IsSceneWithCursor () {

	return  (mPackageID == PackageID.mainMenuScene || mPackageID == PackageID.pauseMenuScene || mPackageID == PackageID.singlePlayerModesMenuScene || mPackageID == PackageID.displayOrientationMenuScene);

}


public boolean IsForcedNumlockNeeded () {


	return  true;



}


public boolean IsSceneWithSoftKey () {

	return  !IsSplashMenuScenePackage() && mPackageID != PackageID.emptyScene;

}


public boolean IsSceneWithDynamicSoftkeyPosition() {
 return  mPackageID != PackageID.gameScene && IsSceneWithSoftKey(); 
}


public boolean IsAppPausedWithFocus() {
 return  GetApp().IsPausedWithFocus(); 
}


public boolean IsSceneWithScramblingAnimation () {

	 short lang = LanguageManager.Get().GetLanguage();


	return  mPackageID != PackageID.gameScene && !IsSplashMenuScenePackage() && mPackageID != PackageID.emptyScene;

}


public boolean IsSplashMenuScenePackage () {

	return  mPackageID == PackageID.splashMenuScene;

}


public void StartEntranceAnimation () {




	if(IsMenuScene())
	{
		
		mpMenuWindowAnimationTimeSystem.RegisterInGlobalTime();
	}


	if(mpScramblingTextTimeSystem != null)
	{
		mpScramblingTextTimeSystem.RegisterInGlobalTime();
	}

}


public boolean IsEntranceAnimationOver () {

	
	 boolean scramblingDone = true;


	scramblingDone = (mpScramblingTextTimeSystem == null || mpScramblingTextTimeSystem.GetTotalTime() >= mScramblingTextDuration);
return  (scramblingDone && (!SupportsTransitionAnimation() || mpMenuWindowAnimationTimeSystem.GetTotalTime() >= (((int)ConstantsEnglishPortrait.menuWindowAnimDurationMs == (int)ConstantsEnglishLandscape.menuWindowAnimDurationMs) ? Constants.menuWindowAnimDurationMs : ConstantsGetter.GetConstants().menuWindowAnimDurationMsFct())));

}


public void StopEntranceAnimation () {




	
	if(IsMenuScene())
	{
		mpMenuWindowAnimationTimeSystem.SetTotalTime((((int)ConstantsEnglishPortrait.menuTextOpenAnimLastFrame == (int)ConstantsEnglishLandscape.menuTextOpenAnimLastFrame) ? Constants.menuTextOpenAnimLastFrame : ConstantsGetter.GetConstants().menuTextOpenAnimLastFrameFct()));
		mpMenuWindowAnimationTimeSystem.OnTime(0, 0);
		mpMenuWindowAnimationTimeSystem.UnRegisterInGlobalTime();
	}


	if(mpScramblingTextTimeSystem != null)
	{
		mpScramblingTextTimeSystem.OnTime(0, (((int)ConstantsEnglishPortrait.animScramblingTextMaxMenuPeriod == (int)ConstantsEnglishLandscape.animScramblingTextMaxMenuPeriod) ? Constants.animScramblingTextMaxMenuPeriod : ConstantsGetter.GetConstants().animScramblingTextMaxMenuPeriodFct()));
		mpScramblingTextTimeSystem.UnRegisterInGlobalTime();
	}

}


public void StartExitAnimation () {




	if(IsSceneWithCursor())
	{
		InitCursorHide();
	}

	
	mpMenuWindowExitAnimationTimeSystem.RegisterInGlobalTime();

}


public void InitCursorShow () {

	 KeyFrameSequence cursorOpenPositionSequence = null;
	 Selector pMenuSelector = null;
	 Selection pMenuSelection = null;

	(pMenuSelector)= (pMenuSelector).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuSelector_ID), null);

	(cursorOpenPositionSequence)= (cursorOpenPositionSequence).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuCursorOpenPositionSequence_ID), null);

	pMenuSelection = pMenuSelector.GetSelectionAt(pMenuSelector.GetSingleSelection());

	 Text pMenuText = null;
	pMenuText = (Text)pMenuSelection.GetChild((((int)ConstantsEnglishPortrait.menuItemTextChildIndex == (int)ConstantsEnglishLandscape.menuItemTextChildIndex) ? Constants.menuItemTextChildIndex : ConstantsGetter.GetConstants().menuItemTextChildIndexFct()));

	 short xAlign = (short)(pMenuText.GetAbsoluteLeft() - (((int)ConstantsEnglishPortrait.cursorOffsetX == (int)ConstantsEnglishLandscape.cursorOffsetX) ? Constants.cursorOffsetX : ConstantsGetter.GetConstants().cursorOffsetXFct()));


	int[] destValues = {-xAlign, 0};
	cursorOpenPositionSequence.SetKeyFrame(0, (short)(((int)ConstantsEnglishPortrait.menuCursorOpenAnimStartMs == (int)ConstantsEnglishLandscape.menuCursorOpenAnimStartMs) ? Constants.menuCursorOpenAnimStartMs : ConstantsGetter.GetConstants().menuCursorOpenAnimStartMsFct()), destValues);

}


public void InitCursorHide () {

	 Viewport pMenuViewport = null;
	 Selector pMenuSelector = null;
	 Selection pMenuSelection = null;







(pMenuSelector)= (pMenuSelector).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuSelector_ID), null);


	(pMenuViewport)= (pMenuViewport).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuScrollingViewport_ID), null);

	
	pMenuSelection = pMenuSelector.GetSelectionAt(pMenuSelector.GetSingleSelection());

	
	 Text pMenuText = null;
	pMenuText = (Text)pMenuSelection.GetChild((((int)ConstantsEnglishPortrait.menuItemTextChildIndex == (int)ConstantsEnglishLandscape.menuItemTextChildIndex) ? Constants.menuItemTextChildIndex : ConstantsGetter.GetConstants().menuItemTextChildIndexFct()));

	 short textWidth = (short)((((int)ConstantsEnglishPortrait.cursorOffsetX == (int)ConstantsEnglishLandscape.cursorOffsetX) ? Constants.cursorOffsetX : ConstantsGetter.GetConstants().cursorOffsetXFct()) + pMenuText.GetFont().GetLineWidth(pMenuText.GetCaption()));


	 short selectorViewportOffsetY = (short)(pMenuViewport.GetOffsetY() - pMenuViewport.GetRectTop());


	 short selectorLeft = (short)((((int)ConstantsEnglishPortrait.screenWidth == (int)ConstantsEnglishLandscape.screenWidth) ? Constants.screenWidth : ConstantsGetter.GetConstants().screenWidthFct()) /2- textWidth /2- (((int)ConstantsEnglishPortrait.cursorOffsetX == (int)ConstantsEnglishLandscape.cursorOffsetX) ? Constants.cursorOffsetX : ConstantsGetter.GetConstants().cursorOffsetXFct()) /2);

	 short selectorTop = (short)(pMenuSelection.GetRectTop() - selectorViewportOffsetY);


	 short selectorOffsetToX = selectorLeft;

	 short selectorOffsetToY = selectorTop;


	 short selectorWidth = textWidth;

	 short selectorHeight = pMenuSelection.GetRectHeight();











{
		pMenuSelector.OffsetTo(selectorOffsetToX, selectorOffsetToY);
		pMenuSelector.SetSize(selectorWidth, selectorHeight);
		pMenuSelector.SetTopLeft((short)(pMenuSelector.GetRectLeft() + selectorLeft), (short)(pMenuSelector.GetRectTop() + selectorTop));
	}
	 KeyFrameSequence cursorHideSequence = null;
	 KeyFrameSequence textHideSizeSequence = null;
	 KeyFrameSequence textHideViewportPositionSequence = null;
	 KeyFrameSequence textHideTextPositionSequence = null;

	(cursorHideSequence)= (cursorHideSequence).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuCursorClosePositionSequence_ID), null);
	(textHideSizeSequence)= (textHideSizeSequence).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuSelectedItemHideSizeAnimSequence_ID), null);
	(textHideViewportPositionSequence)= (textHideViewportPositionSequence).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuSelectedItemHideViewportPositionAnimSequence_ID), null);
	(textHideTextPositionSequence)= (textHideTextPositionSequence).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuSelectedItemHideTextPositionAnimSequence_ID), null);

	 Controller textHidePositionController = null;
	(textHidePositionController)= (textHidePositionController).Cast((mpPackage).GetEntryPoint(RSRC_mainMenuScenePackage.mainMenuSelectedItemHideTextPositionAnimController_ID), null);
	textHidePositionController.SetControlParameters(pMenuSelection, componentPosition);

	int[] destValues = {textWidth, 0};
	int[] destValuesInv = {-textWidth, 0};
	cursorHideSequence.SetKeyFrame(1, (short)(((int)ConstantsEnglishPortrait.menuCursorCloseAnimDuration == (int)ConstantsEnglishLandscape.menuCursorCloseAnimDuration) ? Constants.menuCursorCloseAnimDuration : ConstantsGetter.GetConstants().menuCursorCloseAnimDurationFct()), destValues);
	textHideSizeSequence.SetKeyFrame(1, (short)(((int)ConstantsEnglishPortrait.menuCursorCloseAnimDuration == (int)ConstantsEnglishLandscape.menuCursorCloseAnimDuration) ? Constants.menuCursorCloseAnimDuration : ConstantsGetter.GetConstants().menuCursorCloseAnimDurationFct()), destValuesInv);
	textHideViewportPositionSequence.SetKeyFrame(1, (short)(((int)ConstantsEnglishPortrait.menuCursorCloseAnimDuration == (int)ConstantsEnglishLandscape.menuCursorCloseAnimDuration) ? Constants.menuCursorCloseAnimDuration : ConstantsGetter.GetConstants().menuCursorCloseAnimDurationFct()), destValues);
	textHideTextPositionSequence.SetKeyFrame(1, (short)(((int)ConstantsEnglishPortrait.menuCursorCloseAnimDuration == (int)ConstantsEnglishLandscape.menuCursorCloseAnimDuration) ? Constants.menuCursorCloseAnimDuration : ConstantsGetter.GetConstants().menuCursorCloseAnimDurationFct()), destValuesInv);

}


public boolean IsExitAnimationOver () {

	
	return  (!IsMenuScene() || mpMenuWindowExitAnimationTimeSystem.GetTotalTime() >= (((int)ConstantsEnglishPortrait.menuExitAnimDurationMs == (int)ConstantsEnglishLandscape.menuExitAnimDurationMs) ? Constants.menuExitAnimDurationMs : ConstantsGetter.GetConstants().menuExitAnimDurationMsFct()));

}


public void StopExitAnimation () {




	
	if(IsMenuScene())
	{
		mpMenuWindowExitAnimationTimeSystem.UnRegisterInGlobalTime();
	}

}


public boolean IsReadyToStartEntranceAnim() {
 return  mState == kWaitingOneOnTime; 
}


public void OnSceneReady () {


	if(mPackageID == PackageID.emptyScene)
	{
		RessourcesManager.Get().ReloadMultiTargetPackages();
		HandleCommand(GetExitSceneCommand()); 
	}

}


public static TetrisApp GetApp() {
 return  TetrisApp.Get(); 
}


public TetrisState GetTetrisState() {
 return  mTetrisState; 
}


public TetrisGame GetPlayerGameState (byte player) {

	return  GetTetrisState().GetPlayerGameState(player);

}


public TetrisGame GetPlayer1GameState() {
 return  GetPlayerGameState(TetrisState.kPlayer1); 
}


public TetrisGame GetPlayer2GameState() {
 return  GetPlayerGameState(TetrisState.kPlayer2); 
}


public boolean IsKeyMappedToAcceptSoftkey (int key) {

	return  (key == Key.SOFT_SELECT || key == Key.FIRE);

}


public byte mState;
public TetrisState mTetrisState;
public int mCommandUsedToInstantiate;
public Selection[] mpSoftKeySelection = new Selection[softKeyCount];

public Text[] mpSoftKeyText = new Text[softKeyCount];

public Package mpPackage;
public Viewport mpViewport;
public Component mpComponentWithFocus;
public int mExitSceneTransitionCommand;
public TimeSystem mpMenuWindowAnimationTimeSystem;
public TimeSystem mpMenuWindowExitAnimationTimeSystem;
public TimerSequence mpMenuWindowBorderTimerSequence;
public TimeSystem mpScramblingTextTimeSystem;
public int mScramblingTextDuration;
public boolean isPenOnComponent (Component componentToVerify, Vector2_short inPoint) {

	return  isPenOnComponent(componentToVerify, inPoint, 0, 0, 0, 0);

}


public boolean isPenOnComponentDoubleSize (Component componentToVerify, Vector2_short inPoint) {

	return  isPenOnComponent(componentToVerify, inPoint, componentToVerify.GetRectHeight(), componentToVerify.GetRectHeight(), componentToVerify.GetRectWidth(), componentToVerify.GetRectWidth());

}


public boolean isPenOnComponent (Component componentToVerify, Vector2_short inPoint, int expandValUp, int expandValDown, int expandValLeft, int expandValRight) {

	if(componentToVerify != null)
	{	
		 FlRect rectCurrentSelection = new FlRect(componentToVerify.GetRect());
	

		 short myRect_left = (short)(componentToVerify.GetAbsoluteLeft() - expandValLeft);
  short myRect_top = (short)(componentToVerify.GetAbsoluteTop() - expandValUp);
  short myRect_width = (short)(rectCurrentSelection.GetWidth() + expandValLeft + expandValRight);
  short myRect_height = (short)(rectCurrentSelection.GetHeight() + expandValUp + expandValDown);
				
if(FlRect.Contains(myRect_left, myRect_top, myRect_width, myRect_height, (short)inPoint.GetX(), (short)inPoint.GetY()))
		{
			return  true;	
		}
	}
	return  false;

}


public void OnPenDown ( Vector2_short ptIn) {



}


public void OnPenUp ( Vector2_short ptIn) {

	
	if(mUpArrowSelection != null && mDownArrowSelection != null)
	{
		if(isPenOnComponent(mUpArrowSelection, ptIn))
		{
			ScrollUp();
		}
		else if(isPenOnComponent(mDownArrowSelection, ptIn))
		{
			ScrollDown();
		}
	}
	
	 int leftSoftKeyKey = Key.CLEAR;

	 int rightSoftKeyKey = Key.SOFT_SELECT;



	leftSoftKeyKey = Key.SOFT_SELECT;
	rightSoftKeyKey = Key.CLEAR;
if(isPenOnComponent(mpSoftKeySelection[acceptSoftKey], ptIn, mpSoftKeySelection[acceptSoftKey].GetRectHeight(), mpSoftKeySelection[acceptSoftKey].GetRectHeight(), 0, 0))
	{
		
		if(ptIn.GetX() < (((int)ConstantsEnglishPortrait.screenWidth == (int)ConstantsEnglishLandscape.screenWidth) ? Constants.screenWidth : ConstantsGetter.GetConstants().screenWidthFct()) /2)
		{
			OnKey(leftSoftKeyKey, false);
			OnKey(leftSoftKeyKey, true);
		}
		else
		{
			OnKey(rightSoftKeyKey, false);
			OnKey(rightSoftKeyKey, true);
		}
	}

}


public void ScrollUp () {

	GetApp().OnKeyFromOS(Key.UP, false);
	GetApp().OnKeyFromOS(Key.UP, true);

}


public void ScrollDown () {

	GetApp().OnKeyFromOS(Key.DOWN, false);
	GetApp().OnKeyFromOS(Key.DOWN, true);

}


public Selection mUpArrowSelection;
public Selection mDownArrowSelection;
public byte mPackageID;
public boolean mSaveOnExit;
public boolean mSkipScene;
public void ChangeFont(Text currentText, boolean enable, boolean selected)
{
	ChangeFont(currentText, enable, selected, false);
}



}

