package ca.jamdat.tetris;

import ca.jamdat.flight.*;
import ca.jamdat.fuser.*;
import ca.jamdat.flight.String;
import ca.jamdat.flight.Package;


public class FileHandler extends TimeControlled {

public final static short stateClosed = 0;
public final static short stateCheckingExistence = 1;
public final static short stateCreatingFile = 2;
public final static short stateOpeningReadMode = 3;
public final static short stateOpeningWriteMode = 4;
public final static short stateReady = 5;
public final static short stateClosing = 6;
public final static short stateWriting = 7;
public final static short stateReading = 8;
public final static short stateDeleting = 9;

public final static short statusNotEnoughSpace = 0;
public final static short statusNotFound = 1;
public final static short statusCorruptedOrVersionMismatch = 2;
public final static short statusReady = 3;
public final static short statusWorking = 4;
public final static short statusDeleted = 5;
public final static short statusUnknown = 6;

public  FileHandler () {
mState = (stateClosed);
mStatus = (statusUnknown);





}


public void destruct() {

	Terminate();

	for(int i = 0;i < mSegmentCount;i ++)
	{
		mSegmentStreams[i] = null;
	}

	mSegmentStreams = null;
	mFileStream = null;
	mPathname = null;

}


public void Initialize ( String pathname, int[] segmentSize, int segmentCount, int version) {




	mPathname = new String(pathname);
	mSegmentCount = (byte)segmentCount;
	mVersion = version;

	mSegmentStreams = new FileSegmentStream[segmentCount];



	for(int i = 0;i < segmentCount;i ++)
	{
		mSegmentStreams[i] = new FileSegmentStream();
		mSegmentStreams[i].Initialize(segmentSize[i]);
	}

}


public void Terminate () {



if(mFileStream != null)
	{




mFileStream.Close();
}

}


public void SetPathname ( String pathname) {


	mPathname.Assign(pathname);

}


public String GetPathname () {


	return mPathname;

}


public int GetSize () {



 int sizeInBytes = 4 + 4;


	for(int i = 0;i < mSegmentCount;i ++)
	{
		sizeInBytes += mSegmentStreams[i].GetSize();
	}

	return  sizeInBytes;

}


public int GetWritingCount() {
 return  mWritingCount; 
}


public short GetStatus () {

	return  mStatus;

}


public void StartReadingProcess () {





	Clear();

	SetState(stateCheckingExistence);
	SetStatus(statusWorking);

	RegisterOnTime();

}


public void StartCreationProcess () {





	Clear();

	SetState(stateCreatingFile);
	SetStatus(statusWorking);

	RegisterOnTime();

}


public void StartWritingProcess () {



	if(IsModified())
	{


		SetStatus(statusWorking);
		SetState(stateOpeningWriteMode);

		Open(FileStream.fileModeWrite);
		RegisterOnTime();
	}
	else
	{

		SetStatus(statusReady);
	}

}


public FileSegmentStream GetInputSegmentStream (int index) {


	mSegmentStreams[index].SetMode(FileSegmentStream.modeRead);
	return  mSegmentStreams[index];

}


public FileSegmentStream GetOutputSegmentStream (int index) {


	mSegmentStreams[index].SetMode(FileSegmentStream.modeWrite);
	return  mSegmentStreams[index];

}


public void OnTime (int totalTimeMs, int deltaTimeMs) {






	if(mState == stateCheckingExistence)
	{
		OnCheckingFileExistence();
	}

	if(mState == stateCreatingFile)
	{
		OnCreatingFile();
	}

	if(mState == stateOpeningReadMode)
	{
		OnOpeningFileReadMode();
	}

	if(mState == stateReading)
	{
		OnReadingFile();
	}

	if(mState == stateOpeningWriteMode)
	{
		OnOpeningFileWriteMode();
	}

	if(mState == stateWriting)
	{
		OnWritingFile();
	}





if(mState == stateClosing)
	{
		OnClosingFile();
	}

	if(mState == stateClosed)
	{
		UnRegisterInGlobalTime();
	}

}


public void Open (byte openMode) {














mFileStream = new FileStream(mPathname, openMode, GetSize());

}


public void Clear () {





	for(int i = 0;i < mSegmentCount;i ++)
	{
		mSegmentStreams[i].SetValidDataFlag(false);
	}

}


public boolean IsModified () {

	for(int i = 0;i < mSegmentCount;i ++)
	{
		if(mSegmentStreams[i].IsModified())
		{
			return  true;
		}
	}

	return  false;

}


public int ReadWritingCount () {


	return  mFileStream.ReadLong();

}


public void WriteWritingCount () {


	mFileStream.WriteLong(++mWritingCount);

}


public int ReadVersion () {



	return  mFileStream.ReadLong();

}


public void WriteVersion () {



	mFileStream.WriteLong(mVersion);

}


public void ReadSegments () {



	for(int i = 0;i < mSegmentCount;i ++)
	{

		mSegmentStreams[i].Read(mFileStream);
	}

}


public void WriteSegments () {



	for(int i = 0;i < mSegmentCount;i ++)
	{

		mSegmentStreams[i].Write(mFileStream);
	}

}


public void OnCheckingFileExistence () {






















if(FileStream.FileExists(mPathname))
	{

		SetState(stateOpeningReadMode);
		Open(FileStream.fileModeRead);
	}
	else
	{

		SetStatus(statusNotFound);
		SetState(stateClosed);
	}

}


public void OnCreatingFile () {

















SetState(stateOpeningWriteMode);
	 Open(FileStream.fileModeWrite);

}


public void OnWritingFile () {





	WriteVersion();
	WriteWritingCount();
	WriteSegments();

	mFileStream.Close();
	SetState(stateClosing);

}


public void OnReadingFile () {





	if(mFileStream.IsValid())
	{
		 int version = ReadVersion();

		mWritingCount = ReadWritingCount();

		if(version == mVersion)
		{
			ReadSegments();
		}
		else
		{

			Clear();
			SetStatus(statusCorruptedOrVersionMismatch);
		}
	}
	else
	{

		Clear();
		SetStatus(statusCorruptedOrVersionMismatch);
	}

	mFileStream.Close();
	SetState(stateClosing);

}


public void OnOpeningFileReadMode () {





if(mFileStream.IsOpen())
{

		SetState(stateReading);
	}

}


public void OnOpeningFileWriteMode () {



	if(mFileStream.IsOpen())
	{

		SetState(stateWriting);
	}

}


public void OnClosingFile () {








	if(!mFileStream.IsOpen())
{
		if(GetStatus() == statusWorking)
		{
			SetStatus(statusReady);
		}



		mFileStream = null;
		SetState(stateClosed);
	}

}


public void RegisterOnTime () {

	this.OnTime(0, 0);






}


public void SetStatus(short nextStatus) {
 mStatus = nextStatus; 
}


public void SetState (short nextState) {

	mState = nextState;

}


public String mPathname;
public FileStream mFileStream;
public FileSegmentStream[] mSegmentStreams = null;
public short mState;
public short mStatus;
public int mSegmentCount;
public int mVersion;
public int mWritingCount;

/* One dimension array */
public static FileHandler[] InstArrayFileHandler(int size) {
	FileHandler[] newArray = new FileHandler[size];
	for(int i = 0 ; i < size ; i++) newArray[i] = new FileHandler();
	return newArray; }

/* Two dimension array */
public static FileHandler[][] InstArrayFileHandler(int size1, int size2) {
	FileHandler[][] newArray = new FileHandler[size1][];
	for(int i = 0 ; i < size1 ; i++) {
		newArray[i] = new FileHandler[size2];
		for(int j = 0 ; j < size2 ; j++) newArray[i][j] = new FileHandler(); }
	return newArray; }

/* Three dimension array */
public static FileHandler[][][] InstArrayFileHandler(int size1, int size2, int size3) {
	FileHandler[][][] newArray = new FileHandler[size1][][];
	for(int i = 0 ; i < size1 ; i++) {
		newArray[i] = new FileHandler[size2][];
		for(int j = 0 ; j < size2 ; j++) {
			newArray[i][j] = new FileHandler[size3];
			for(int k = 0 ; k < size3 ; k++) newArray[i][j][k] = new FileHandler(); } }
	return newArray; }



}

