package ca.jamdat.tetris;

import ca.jamdat.flight.*;
import ca.jamdat.fuser.*;
import ca.jamdat.flight.String;
import ca.jamdat.flight.Package;


public class FileSegmentStream {

public final static short modeRead = 0;
public final static short modeWrite = 1;

public  FileSegmentStream () {
mModified = (true);
mMode = (modeRead);



}


public void destruct() {

	mByteArray = null;

}


public void Initialize (int capacityInBytes) {


	mByteArrayCapacity = capacityInBytes;
	mByteArray = new byte[capacityInBytes];




}


public void SetValidDataFlag (boolean valid) {

	 boolean invalidating = (mValidData == true && valid == false);

	mModified = (invalidating || mModified);
	mValidData = valid;

}


public boolean HasValidData () {

	return  mValidData;

}


public boolean IsModified () {

	return  mModified;

}


public void WriteByte (byte value) {


	
	if(mByteArray[mPosition] != value)
	{
		mModified = true;
		mByteArray[mPosition] = value;
	}
	
	mPosition++;
	


}


public void WriteBoolean (boolean value) {



	if(value)
	{
		WriteByte(((byte)1));
	}
	else
	{
		WriteByte(((byte)0));
	}

}


public void WriteShort (short value) {



	WriteByte((byte)((value & 0xFF00) >> 8));
	WriteByte((byte)((value & 0x00FF)));



}


public void WriteInt (int value) {



	WriteShort((short)((value & 0xFFFF0000) >> 16));
	WriteShort((short)((value & 0x0000FFFF)));



}


public void WritePaddedString ( String s, int maxLength) {



	 int sLen = s.GetLength();


	 int paddingCharCount = maxLength - sLen;


	 String stringToWrite = new String(s);

	for(int i = 0;i < paddingCharCount;i ++)
	{
		
		stringToWrite.AddAssign(StringToolbox.ZeroString());
	}

	WriteInt(sLen);
	WriteString(stringToWrite);

	mModified = true;

}


public void WriteF32 ( F32 fp) {

	WriteInt(fp.ToFixedPoint(16));

}


public void WriteFVec3 ( FVec3T_F32 vec3) {

	WriteF32(vec3.x);
	WriteF32(vec3.y);
	WriteF32(vec3.z);

}


public byte ReadByte () {


	return  mByteArray[mPosition++];

}


public boolean ReadBoolean () {


	return  ReadByte() == ((byte)1);

}


public short ReadShort () {



	 byte b1 = ReadByte();

	 byte b2 = ReadByte();

	 short s1 = (short)((b1 << 8) & 0xFF00);

	 short s2 = (short)(b2 & 0x00FF);

	 short read = (short)(s1 | s2);




	return  read;

}


public int ReadInt () {



	 short s1 = ReadShort();

	 short s2 = ReadShort();

	 int i1 = (int)((s1 << 16) & 0xffff0000);

	 int i2 = (int)((s2 & 0x0000ffff));

	 int read = (i1 | i2);




	return  read;

}


public String ReadPaddedString () {


	 int sLen = ReadInt();

	String str = new String();

	ReadString(str);
	return  str.Substring(0, sLen);

}


public F32 ReadF32 () {

	return  new F32((int)ReadInt(), 16);

}


public FVec3T_F32 ReadFVec3 () {

	 F32 x = new F32(ReadF32());
 
	 F32 y = new F32(ReadF32());
 
	 F32 z = new F32(ReadF32());
 
	return  new FVec3T_F32(x, y, z);

}


public static void Test () {




































}


public void SetMode (short ioMode) {

	if(ioMode != mMode)
	{
		mMode = ioMode;
	}

	
	SetPosition(0);

}


public void SetPosition (int position) {


	mPosition = (short)position;

}


public int GetPosition () {

	return  mPosition;

}


public int GetSize () {

	
	return  1 + mByteArrayCapacity;

}


public void Read (FileStream stream) {



	SetMode(FileSegmentStream.modeRead);

	 boolean validData = (stream.ReadByte() == (byte)1);

	SetValidDataFlag(validData);
	stream.Read(mByteArray, mByteArrayCapacity);

	
	mModified = false;

}


public void Write (FileStream stream) {

	
	



	SetMode(FileSegmentStream.modeWrite);

	 byte validData = HasValidData() ? (byte)1 : (byte)0;

	stream.WriteByte(validData);
	stream.Write(mByteArray, mByteArrayCapacity);

	
	mModified = false;

}


public void ReadString (String sRet) {


	
	 int length = ReadInt();

	for(int i = 0;i < length;i ++)
	{
		sRet.AddAssign(String.FromChar((byte)ReadShort()));
	}

}


public void WriteString ( String s) {


	
	 int length = s.GetLength();

	WriteInt(length);
	for(int i = 0;i < length;i ++)
	{
		WriteShort((short)s.GetCharAt(i));
	}

}


public int mByteArrayCapacity;
public byte[] mByteArray = null;
public int mPosition;
public boolean mValidData;
public boolean mModified;
public short mMode;

/* One dimension array */
public static FileSegmentStream[] InstArrayFileSegmentStream(int size) {
	FileSegmentStream[] newArray = new FileSegmentStream[size];
	for(int i = 0 ; i < size ; i++) newArray[i] = new FileSegmentStream();
	return newArray; }

/* Two dimension array */
public static FileSegmentStream[][] InstArrayFileSegmentStream(int size1, int size2) {
	FileSegmentStream[][] newArray = new FileSegmentStream[size1][];
	for(int i = 0 ; i < size1 ; i++) {
		newArray[i] = new FileSegmentStream[size2];
		for(int j = 0 ; j < size2 ; j++) newArray[i][j] = new FileSegmentStream(); }
	return newArray; }

/* Three dimension array */
public static FileSegmentStream[][][] InstArrayFileSegmentStream(int size1, int size2, int size3) {
	FileSegmentStream[][][] newArray = new FileSegmentStream[size1][][];
	for(int i = 0 ; i < size1 ; i++) {
		newArray[i] = new FileSegmentStream[size2][];
		for(int j = 0 ; j < size2 ; j++) {
			newArray[i][j] = new FileSegmentStream[size3];
			for(int k = 0 ; k < size3 ; k++) newArray[i][j][k] = new FileSegmentStream(); } }
	return newArray; }



}

