package ca.jamdat.tetris;

import ca.jamdat.flight.*;
import ca.jamdat.fuser.*;
import ca.jamdat.flight.String;
import ca.jamdat.flight.Package;


public abstract class InputControler extends TimeControlled {

public final static int REPEAT_IN_PRESSED_NO_REPEAT = -1;

public  InputControler () {




}


public void Destruct () {

	SetEnabled(false);

}


public boolean IsEnabled() {
 return  mEnabled; 
}


public void SetEnabled (boolean enabled) {

	if(enabled == IsEnabled())
	{
		return ; 
	}
	mEnabled = enabled;

	if(enabled)
	{
		OnEnabled();
	}
	else
	{
		OnDisabled();
	}

}


public void OnTime (int a7, int deltaTimeMs) {

	 InputMapping mapping = GetMapping();

	
	 int mappingCount = mapping.GetMappingCount();

	int i;

	for(i = 0;i < mappingCount;i ++)
	{
		 int repeatIn = mapping.GetMappingField(i, InputMapping.kMappingKeyRepeatIn);

		if(repeatIn <= 0)
		{
			continue; 
		}

		repeatIn -= deltaTimeMs;
		if(repeatIn <= 0)
		{
			
			 int stepDelay = mapping.GetMappingField(i, InputMapping.kMappingKeyRepeatStepDelay);

			 int actionId = mapping.GetMappingField(i, InputMapping.kMappingAction);


			for(int nthRepeat = 1;repeatIn <= 0;nthRepeat ++)
			{
				repeatIn += stepDelay;				
				OnActionRepeat(actionId, nthRepeat); 
			}
		}

		mapping.SetMappingField(i, InputMapping.kMappingKeyRepeatIn, repeatIn);
	}

	
	 int actionCount = mapping.GetActionCount();

	for(i = 0;i < actionCount;i ++)
	{
		 int repeatIn = mapping.GetActionField(i, InputMapping.kActionRepeatIn);

		if(repeatIn <= 0)
		{
			continue; 
		}

		repeatIn -= deltaTimeMs;
		if(repeatIn <= 0)
		{
			
			 int stepDelay = mapping.GetActionField(i, InputMapping.kActionRepeatStepDelay);


			for(int nthRepeat = 1;repeatIn <= 0;nthRepeat ++)
			{
				repeatIn += stepDelay;				
				OnActionRepeat(i, nthRepeat); 
			}
		}

		mapping.SetActionField(i, InputMapping.kActionRepeatIn, repeatIn);
	}

}


public void OnKey (int key, boolean up) {

	if(!mEnabled && !up)
	{
		return ; 
	}

	 InputMapping mapping = GetMapping();
	 int actionId = mapping.GetMappingFieldForKey(key, InputMapping.kMappingAction);


	 int pressCountVar = up ? -1 : 1;

	 int pressCount = mapping.GetActionField(actionId, InputMapping.kActionKeyDownCounter) + pressCountVar;

	if(pressCount < 0)
	{
		return ; 
	}

	mapping.SetActionField(actionId, InputMapping.kActionKeyDownCounter, pressCount);

	
	
	if(!up && pressCount == 1) 
	{
		 int initialDelay = mapping.GetActionField(actionId, InputMapping.kActionRepeatInitialDelay);

		mapping.SetActionField(actionId, InputMapping.kActionRepeatIn, initialDelay);
	}
	else if(up && pressCount == 0) 
	{
		mapping.SetActionField(actionId, InputMapping.kActionRepeatIn, 0);
	}

	
	{
		int repeatInMs;

		if(up)
		{
			repeatInMs = 0;
		}
		else
		{
			repeatInMs = mapping.GetMappingFieldForKey(key, InputMapping.kMappingKeyRepeatInitialDelay);
			if(repeatInMs == 0)
			{
				repeatInMs = REPEAT_IN_PRESSED_NO_REPEAT;
			}
		}
		mapping.SetMappingFieldForKey(key, InputMapping.kMappingKeyRepeatIn, repeatInMs);
	}

	if(mEnabled)
	{
		if(up)
		{
			OnActionReleased(actionId, pressCount);
		}
		else
		{
			OnActionPressed(actionId, pressCount);
		}
	}

}


public void ReleaseAllKeys () {

	 boolean sendMessage = IsEnabled();

	 InputMapping mapping = GetMapping();
	int i;

	
	for(i = 0;i < mapping.GetMappingCount();i ++)
	{
		 int repeatIn = mapping.GetMappingField(i, InputMapping.kMappingKeyRepeatIn);

		mapping.SetMappingField(i, InputMapping.kMappingKeyRepeatIn, 0);
		
		 boolean keyIsPressed = repeatIn == REPEAT_IN_PRESSED_NO_REPEAT && repeatIn > 0;

		if(!keyIsPressed)
		{
			continue;
		}

		 int actionId = mapping.GetMappingField(i, InputMapping.kMappingAction);

		 int pressCount = mapping.GetActionField(actionId, InputMapping.kActionKeyDownCounter);


		if(pressCount > 0)
		{
			pressCount--;
			mapping.SetActionField(actionId, InputMapping.kActionKeyDownCounter, pressCount);

			if(pressCount == 0)
			{
				mapping.SetActionField(actionId, InputMapping.kActionRepeatIn, 0);
			}
		}

		if(sendMessage)
		{
			OnActionReleased(actionId, pressCount);
		}
	}

	
	for(i = 0;i < mapping.GetActionCount();i ++)
	{
		 int keyDown = mapping.GetActionField(i, InputMapping.kActionKeyDownCounter);

		if(keyDown == 0)
		{
			continue; 
		}

		while(keyDown -- > 0)
		{
			OnActionReleased(i, keyDown);
		}

		mapping.SetActionField(i, InputMapping.kActionKeyDownCounter, 0);
		mapping.SetActionField(i, InputMapping.kActionRepeatIn, 0);
	}

}


public abstract void OnActionRepeat(int a8, int a7);



public abstract void OnActionPressed(int a10, int a9);



public abstract void OnActionReleased(int a12, int a11);



public abstract InputMapping GetMapping();



public void OnEnabled() {
 RegisterInGlobalTime(); 
}


public void OnDisabled() {
 UnRegisterInGlobalTime(); 
}


public boolean mEnabled;



}

