package ca.jamdat.tetris;

import ca.jamdat.flight.*;
import ca.jamdat.fuser.*;
import ca.jamdat.flight.String;
import ca.jamdat.flight.Package;


public abstract class Tetrimino extends TimeControlled {

public final static int kTetriminoI = 0;
public final static int kTetriminoJ = 1;
public final static int kTetriminoL = 2;
public final static int kTetriminoO = 3;
public final static int kTetriminoS = 4;
public final static int kTetriminoT = 5;
public final static int kTetriminoZ = 6;
public final static int kTetriminoCount = 7;

public final static int kPieceFaceN = 0;
public final static int kPieceFaceE = 1;
public final static int kPieceFaceS = 2;
public final static int kPieceFaceW = 3;
public final static int kPieceFaceCount = 4;

public  Tetrimino (Well pWell) {
mpWell = pWell;
mFacingDir = (kPieceFaceN);
mSoftDropping = ((00) != 0);


	RegisterInGlobalTime();

	RessourcesManager.Get().GetPackage(PackageID.tetriminoMarks);

	MoveAtStartPosition();
	ForceDefaultOrientationFacing();

}


public void Destruct () {

	UnRegisterInGlobalTime();

	if((mpPoolNext)!= null) { (mpPoolNext).Destruct(); (mpPoolNext)= null; ; };

}


public boolean IndexInBound (int x, int y) {

	return (x >= 0&& x < (((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : ConstantsGetter.GetConstants().minosPerTetriminoFct())) &&
		   (y >= 0&& y < (((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : ConstantsGetter.GetConstants().minosPerTetriminoFct()));

}


public void Assign ( Tetrimino tetrimino) {

	
	
	mFacingDir = tetrimino.mFacingDir;
	mMatrixPosX = tetrimino.mMatrixPosX;
	mMatrixPosY = tetrimino.mMatrixPosY;

	for(int mino = 0;mino < (((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : ConstantsGetter.GetConstants().minosPerTetriminoFct());mino ++)
	{
		mMinosX[mino] = tetrimino.mMinosX[mino];
		mMinosY[mino] = tetrimino.mMinosY[mino];

		mSticky[mino] = tetrimino.mSticky[mino];
}

}


public static Tetrimino CreateTetrimino (int newPieceType, Well pWell) {

	 Tetrimino newTetrimino = null;

	switch(newPieceType)
	{
	case Tetrimino.kTetriminoI : newTetrimino = new TetriminoI(pWell);	break;
	case Tetrimino.kTetriminoJ : newTetrimino = new TetriminoJ(pWell);	break;
	case Tetrimino.kTetriminoL : newTetrimino = new TetriminoL(pWell);	break;
	case Tetrimino.kTetriminoO : newTetrimino = new TetriminoO(pWell);	break;
	case Tetrimino.kTetriminoS : newTetrimino = new TetriminoS(pWell);	break;
	case Tetrimino.kTetriminoT : newTetrimino = new TetriminoT(pWell);	break;
	case Tetrimino.kTetriminoZ : newTetrimino = new TetriminoZ(pWell);	break;
	default :

	}


	newTetrimino.InitSticky();
return  newTetrimino;

}


public static void CopyTetrimino (Tetrimino dest, Tetrimino source, Well pDestWell) {
	


	
	dest.Assign(source);
	dest.mpWell = pDestWell == null ? source.mpWell : pDestWell;

}


public abstract boolean IsFree(int a8, int a7, int a6);



public abstract boolean CanFall();



public abstract int GetType();



public abstract byte GetMinoColor();



public boolean Fall () {



	
	if(!CanFall())
	{
		return  false;
	}

	mMatrixPosY++;

	return  true;

}


public short GetMatrixPosX() {
 return  mMatrixPosX; 
}


public short GetMatrixPosY() {
 return  mMatrixPosY; 
}


public void SetMatrixPos(short x, short y) {
 mMatrixPosX = x; mMatrixPosY = y; 
}


public int GetMinoX (int index) {

	switch((int)mFacingDir)
	{
		case Tetrimino.kPieceFaceN :
			return  mMinosX[index];
		case Tetrimino.kPieceFaceE :
			return  -mMinosY[index];
		case Tetrimino.kPieceFaceS :
			return  -mMinosX[index];
		case Tetrimino.kPieceFaceW :
			return  mMinosY[index];
	}

	return  0;

}


public int GetMinoY (int index) {

	switch((int)mFacingDir)
	{
		case Tetrimino.kPieceFaceN :
			return  mMinosY[index];
		case Tetrimino.kPieceFaceE :
			return  mMinosX[index];
		case Tetrimino.kPieceFaceS :
			return  -mMinosY[index];
		case Tetrimino.kPieceFaceW :
			return  -mMinosX[index];
	}

	return  0;

}


public byte GetSticky(int mino, int facing) {
 return  (byte)((mSticky[mino] >> facing) & 0x0f); 
}


public int GetFacingDir() {
 return  mFacingDir; 
}


public void SetFacingDir(int facing) {
 mFacingDir = facing; 
}


public void ForceDefaultOrientationFacing() {
 mFacingDir = kPieceFaceN; 
}


public boolean Move (boolean left) {

	 boolean validMove = false;

	 int adj = left ? -1 : 1;


	if(IsFree(adj, 0, mFacingDir))
	{
		mMatrixPosX = (short)(mMatrixPosX + adj);
		validMove = true;
	}



return  validMove;

}


public boolean Rotate (boolean clockWise) {

	boolean validRotation;

	
	validRotation = (clockWise ?
					ChangeDir(mFacingDir >= kPieceFaceCount - 1? (int)0: (int) (mFacingDir + 1)) :
					ChangeDir(mFacingDir <= 0? (int)(kPieceFaceCount - 1) : (int) (mFacingDir - 1)));



return  validRotation;

}


public boolean Lock () {

	 boolean lockout = true;

	mLocked = true;

	for(int mino = 0;mino < (((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : ConstantsGetter.GetConstants().minosPerTetriminoFct());mino ++)
	{
		 int column = mMatrixPosX + GetMinoX(mino);

		 int line = mMatrixPosY + GetMinoY(mino);




 byte sticky = GetSticky(mino, mFacingDir);

		mpWell.GetLine(line).SetBlock(column, GetMinoColor(), sticky);
mpWell.NoticeAddedMino(column, line);

		if(line >= (((int)ConstantsEnglishPortrait.wellSkyline == (int)ConstantsEnglishLandscape.wellSkyline) ? Constants.wellSkyline : ConstantsGetter.GetConstants().wellSkylineFct())) 
		{
			lockout = false;
		}
	}

	return  lockout;

}


public void LockAI () {

	for(int mino = 0;mino < (((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : ConstantsGetter.GetConstants().minosPerTetriminoFct());mino ++)
	{
		 int column = mMatrixPosX + GetMinoX(mino);

		 int line = mMatrixPosY + GetMinoY(mino);


		mpWell.GetLine(line).SetBlockAI(column);

		mpWell.NoticeAddedMino(column, line);
	}

}


public void UnlockAI () {

	for(int mino = 0;mino < (((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : ConstantsGetter.GetConstants().minosPerTetriminoFct());mino ++) 
	{
		 int column = mMatrixPosX + GetMinoX(mino);

		 int line = mMatrixPosY + GetMinoY(mino);


		mpWell.GetLine(line).ClearBlockAI(column);

		mpWell.NoticeRemovedMino(column, line);
	}

}


public void MoveAtStartPosition () {

	mMatrixPosX = (short)(((int)ConstantsEnglishPortrait.pieceStartX == (int)ConstantsEnglishLandscape.pieceStartX) ? Constants.pieceStartX : ConstantsGetter.GetConstants().pieceStartXFct());
	mMatrixPosY = (short)(((int)ConstantsEnglishPortrait.pieceStartY == (int)ConstantsEnglishLandscape.pieceStartY) ? Constants.pieceStartY : ConstantsGetter.GetConstants().pieceStartYFct());
	mSoftDropping = false;
	mLocked = false;

}


public void GetLowestMinos (int lowestMinos[] , int clearedLineFlags) {

	for(int i = 0;i < (((int)ConstantsEnglishPortrait.wellWidth == (int)ConstantsEnglishLandscape.wellWidth) ? Constants.wellWidth : ConstantsGetter.GetConstants().wellWidthFct());i ++)
	{
		lowestMinos[i] = -1;
	}

	for(int mino = 0;mino < (((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : ConstantsGetter.GetConstants().minosPerTetriminoFct());mino ++)
	{
		 int column = GetMatrixPosX() + GetMinoX(mino);

		 int row = GetMatrixPosY() + GetMinoY(mino);



		
		if(row > lowestMinos[column] && !Well.IsClearedLineFromFlags(clearedLineFlags, row))
		{
			lowestMinos[column] = row;
		}
	}

}


public boolean IntroducedNewHoles (int clearedLineFlags) {

	for(int mino = 0;mino < (((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : ConstantsGetter.GetConstants().minosPerTetriminoFct());mino ++)
	{
		 int column = GetMatrixPosX() + GetMinoX(mino);

		 int row = GetMatrixPosY() + GetMinoY(mino);


		
		if(!Well.IsClearedLineFromFlags(clearedLineFlags, row))
		{
			
			 int clearedLinesUnderMino = Well.GetClearedLineCountFromFlags(clearedLineFlags, row + 1);

			row += clearedLinesUnderMino;

			if(row + 1< (((int)ConstantsEnglishPortrait.wellHeight == (int)ConstantsEnglishLandscape.wellHeight) ? Constants.wellHeight : ConstantsGetter.GetConstants().wellHeightFct()) && !mpWell.GetLine(row + 1).IsBlockUsed(column))
			{
				return  true;
			}
		}
	}

	return  false;

}


public boolean IsSoftDropping() {
 return  mSoftDropping; 
}


public void SetSoftDropping(boolean softDropping) {
 mSoftDropping = softDropping; 
}


public boolean IsLock() {
 return  mLocked; 
}


public void OnTime (int totalTimeMs, int deltaTimeMs) {




	if(RessourcesManager.Get().IsPackageLoaded(PackageID.tetriminoMarks))
	{
		UnRegisterInGlobalTime();
		mpMarkerArray = GetMarkerArray(GetType() == kTetriminoI ? RSRC_tetriminoMarksPackage.tetriminoIMarks_ID : RSRC_tetriminoMarksPackage.tetriminoJLSTZMarks_ID);
	}

}


public void AddToTetriminoPool (Tetrimino poolRoot) {
   


	mpPoolNext = poolRoot;

}


public Tetrimino GetPooledTetrimino (Well well) {
	
	
	mpWell = well; 
	MoveAtStartPosition();
	ForceDefaultOrientationFacing();
	 Tetrimino newRoot = mpPoolNext;
    mpPoolNext = null;
	return  newRoot;

}


public boolean IsLastInPool() {
 return  mpPoolNext == null; 
}


public boolean ChangeDir (int newFacing) {

	int nx;

	int ny;

	int s;

	int shsx;

	int shsy;

	int dhsx;

	int dhsy;


	for(s = 0;s < (((int)ConstantsEnglishPortrait.tetriminoMarkCount == (int)ConstantsEnglishLandscape.tetriminoMarkCount) ? Constants.tetriminoMarkCount : ConstantsGetter.GetConstants().tetriminoMarkCountFct());s ++)
	{
		
		shsx = GetMarkerX(s, mFacingDir);
		shsy = GetMarkerY(s, mFacingDir);
		dhsx = GetMarkerX(s, newFacing);
		dhsy = GetMarkerY(s, newFacing);
		nx = shsx - dhsx;
		ny = shsy - dhsy;

		
		if(IsFree(nx, ny, newFacing))
		{
			mFacingDir = newFacing;
			mMatrixPosX = (short)(mMatrixPosX + nx);
			mMatrixPosY = (short)(mMatrixPosY + ny);
			return  true;
		}
	}

	return  false;

}


public Array_int GetMarkerArray (int entryPoint) {


 Package marksPackage = RessourcesManager.Get().GetPackage(PackageID.tetriminoMarks, false);

	 Array_int pMarks = null;
	(pMarks)= (pMarks).Cast((marksPackage).GetEntryPoint(entryPoint), null);

	return  pMarks;

}


public int GetMarkerX (int mark_num, int facing) {

	return  mpMarkerArray.GetAt((facing * (((int)ConstantsEnglishPortrait.tetriminoMarkCount == (int)ConstantsEnglishLandscape.tetriminoMarkCount) ? Constants.tetriminoMarkCount : ConstantsGetter.GetConstants().tetriminoMarkCountFct()) + mark_num) * 2);

}


public int GetMarkerY (int mark_num, int facing) {

	return  mpMarkerArray.GetAt((facing * (((int)ConstantsEnglishPortrait.tetriminoMarkCount == (int)ConstantsEnglishLandscape.tetriminoMarkCount) ? Constants.tetriminoMarkCount : ConstantsGetter.GetConstants().tetriminoMarkCountFct()) + mark_num) * 2+ 1);

}


public void InitSticky () {

	boolean[][] stickyCalc = new boolean[(((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : -123)][(((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : -123)];
	int x;

	int y;



	
	for(int i = 0;i < (((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : ConstantsGetter.GetConstants().minosPerTetriminoFct());i ++)
		for(int j = 0;j < (((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : ConstantsGetter.GetConstants().minosPerTetriminoFct());j ++)
			stickyCalc[i][j] = false;

	
	int mino;

	for(mino = 0;mino < (((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : ConstantsGetter.GetConstants().minosPerTetriminoFct());mino ++)
	{
		x = mMinosX[mino] + (((int)ConstantsEnglishPortrait.minoMaxNegativeOffset == (int)ConstantsEnglishLandscape.minoMaxNegativeOffset) ? Constants.minoMaxNegativeOffset : ConstantsGetter.GetConstants().minoMaxNegativeOffsetFct());
		y = mMinosY[mino] + (((int)ConstantsEnglishPortrait.minoMaxNegativeOffset == (int)ConstantsEnglishLandscape.minoMaxNegativeOffset) ? Constants.minoMaxNegativeOffset : ConstantsGetter.GetConstants().minoMaxNegativeOffsetFct());
		stickyCalc[x][y] = true;
	}

	
	
	
	
	
	for(mino = 0;mino < (((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : ConstantsGetter.GetConstants().minosPerTetriminoFct());mino ++)
	{
		x = mMinosX[mino] + (((int)ConstantsEnglishPortrait.minoMaxNegativeOffset == (int)ConstantsEnglishLandscape.minoMaxNegativeOffset) ? Constants.minoMaxNegativeOffset : ConstantsGetter.GetConstants().minoMaxNegativeOffsetFct());
		y = mMinosY[mino] + (((int)ConstantsEnglishPortrait.minoMaxNegativeOffset == (int)ConstantsEnglishLandscape.minoMaxNegativeOffset) ? Constants.minoMaxNegativeOffset : ConstantsGetter.GetConstants().minoMaxNegativeOffsetFct());
		 byte g = 0;


		
		if(IndexInBound(x, y -1) && stickyCalc[x][y-1])
		{
			g |= 0x88;	
		}

		
		if(IndexInBound(x +1, y) && stickyCalc[x+1][y])
		{
			g |= 0x44;	
		}

		
		if(IndexInBound(x, y +1) && stickyCalc[x][y+1])
		{
			g |= 0x22;	
		}

		
		if(IndexInBound(x -1, y) && stickyCalc[x-1][y])
		{
			g |= 0x11;	
		}
		mSticky[mino] = g;
	}

}


public static TetrisApp GetApp () {

	return  TetrisApp.Get();

}


public boolean IsInMatrixRangeY(short y) {
 return  y < (((int)ConstantsEnglishPortrait.wellHeight == (int)ConstantsEnglishLandscape.wellHeight) ? Constants.wellHeight : ConstantsGetter.GetConstants().wellHeightFct()); 
}


public Well mpWell;
public int mFacingDir;
public short mMatrixPosX;
public short mMatrixPosY;
public int[] mMinosX = new int[(((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : -123)];

public int[] mMinosY = new int[(((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : -123)];

public byte[] mSticky = new byte[(((int)ConstantsEnglishPortrait.minosPerTetrimino == (int)ConstantsEnglishLandscape.minosPerTetrimino) ? Constants.minosPerTetrimino : -123)];

public Array_int mpMarkerArray;
public boolean mSoftDropping;
public boolean mLocked;
public Tetrimino mpPoolNext;



}

